"""
Unit tests for the optimizer prompts module.

This module contains tests for the optimization prompts and prompt manager functionality.
"""

from unittest.mock import Mock

import pytest

from src.pagans.optimizer_prompts import (
    BaseOptimizationPrompt,
    OptimizationPromptManager,
    get_optimization_description,
    get_optimization_prompt,
    get_supported_families,
)
from src.pagans.optimizer_prompts.anthropic import (
    ANTHROPIC_OPTIMIZATION_PROMPT,
)
from src.pagans.optimizer_prompts.google import GOOGLE_OPTIMIZATION_PROMPT
from src.pagans.optimizer_prompts.openai import OPENAI_OPTIMIZATION_PROMPT
from src.pagans.optimizer_prompts.xai import XAI_OPTIMIZATION_PROMPT


class TestBaseOptimizationPrompt:
    """Test cases for the BaseOptimizationPrompt class."""

    def test_base_prompt_abstract_methods(self):
        """Test that BaseOptimizationPrompt is abstract."""
        with pytest.raises(TypeError):
            BaseOptimizationPrompt()

    def test_base_prompt_get_prompt_not_implemented(self):
        """Test that abstract class cannot be instantiated without implementing get_prompt."""

        class TestPrompt(BaseOptimizationPrompt):
            def get_description(self):
                return "Test description"

        with pytest.raises(
            TypeError, match="Can't instantiate abstract class TestPrompt"
        ):
            TestPrompt()

    def test_base_prompt_get_description_not_implemented(self):
        """Test that abstract class cannot be instantiated without implementing get_description."""

        class TestPrompt(BaseOptimizationPrompt):
            def get_prompt(self, original_prompt, target_model):
                return "Test prompt"

        with pytest.raises(
            TypeError, match="Can't instantiate abstract class TestPrompt"
        ):
            TestPrompt()


class TestOptimizationPromptManager:
    """Test cases for the OptimizationPromptManager class."""

    def test_init_empty(self):
        """Test initialization with no prompts."""
        manager = OptimizationPromptManager()
        assert len(manager._prompts) == 0

    def test_register_prompt(self):
        """Test registering a prompt."""
        manager = OptimizationPromptManager()
        mock_prompt = Mock()

        manager.register_prompt("test_family", mock_prompt)

        assert "test_family" in manager._prompts
        assert manager._prompts["test_family"] is mock_prompt

    def test_register_prompt_overwrite(self):
        """Test registering a prompt overwrites existing one."""
        manager = OptimizationPromptManager()
        mock_prompt1 = Mock()
        mock_prompt2 = Mock()

        manager.register_prompt("test_family", mock_prompt1)
        manager.register_prompt("test_family", mock_prompt2)

        assert manager._prompts["test_family"] is mock_prompt2

    def test_get_prompt_success(self):
        """Test getting a prompt successfully."""
        manager = OptimizationPromptManager()
        mock_prompt = Mock()
        mock_prompt.get_prompt.return_value = "Test prompt"

        manager.register_prompt("test_family", mock_prompt)

        result = manager.get_prompt("test_family", "original", "target")

        assert result == "Test prompt"
        mock_prompt.get_prompt.assert_called_once_with("original", "target")

    def test_get_prompt_not_found(self):
        """Test getting a prompt that doesn't exist."""
        manager = OptimizationPromptManager()

        with pytest.raises(
            ValueError,
            match="No optimization prompt registered for family: test_family",
        ):
            manager.get_prompt("test_family", "original", "target")

    def test_get_description_success(self):
        """Test getting a description successfully."""
        manager = OptimizationPromptManager()
        mock_prompt = Mock()
        mock_prompt.get_description.return_value = "Test description"

        manager.register_prompt("test_family", mock_prompt)

        result = manager.get_description("test_family")

        assert result == "Test description"
        mock_prompt.get_description.assert_called_once()

    def test_get_description_not_found(self):
        """Test getting a description for a family that doesn't exist."""
        manager = OptimizationPromptManager()

        with pytest.raises(
            ValueError,
            match="No optimization prompt registered for family: test_family",
        ):
            manager.get_description("test_family")

    def test_get_supported_families_empty(self):
        """Test getting supported families when none registered."""
        manager = OptimizationPromptManager()

        result = manager.get_supported_families()

        assert result == []

    def test_get_supported_families_with_prompts(self):
        """Test getting supported families with registered prompts."""
        manager = OptimizationPromptManager()
        mock_prompt1 = Mock()
        mock_prompt2 = Mock()

        manager.register_prompt("family1", mock_prompt1)
        manager.register_prompt("family2", mock_prompt2)

        result = manager.get_supported_families()

        assert result == ["family1", "family2"]

    def test_get_supported_families_no_duplicates(self):
        """Test that supported families have no duplicates."""
        manager = OptimizationPromptManager()
        mock_prompt = Mock()

        manager.register_prompt("family1", mock_prompt)
        manager.register_prompt("family1", mock_prompt)  # Register same family again

        result = manager.get_supported_families()

        assert result == ["family1"]  # No duplicates


class TestFamilySpecificPrompts:
    """Test cases for family-specific optimization prompts."""

    def test_openai_prompt_structure(self):
        """Test that OpenAI prompt has correct structure."""
        prompt = OPENAI_OPTIMIZATION_PROMPT.get_prompt(
            original_prompt="Test prompt", target_model="gpt-5.4"
        )

        assert "OpenAI GPT models" in prompt
        assert "gpt-5.4" in prompt
        assert "Test prompt" in prompt
        assert "Return ONLY the optimized prompt" in prompt
        assert "output schema/format clearly" in prompt

    def test_anthropic_prompt_structure(self):
        """Test that Anthropic prompt has correct structure."""
        prompt = ANTHROPIC_OPTIMIZATION_PROMPT.get_prompt(
            original_prompt="Test prompt", target_model="claude-sonnet-4-20250514"
        )

        assert "Anthropic Claude models" in prompt
        assert "claude-sonnet-4-20250514" in prompt
        assert "Test prompt" in prompt
        assert "Return ONLY the optimized prompt" in prompt
        assert "Use XML tags" in prompt

    def test_google_prompt_structure(self):
        """Test that Google prompt has correct structure."""
        prompt = GOOGLE_OPTIMIZATION_PROMPT.get_prompt(
            original_prompt="Test prompt", target_model="gemini-3.1-pro-preview"
        )

        assert "Google Gemini models" in prompt
        assert "gemini-3.1-pro-preview" in prompt
        assert "Test prompt" in prompt
        assert "Return ONLY the optimized prompt" in prompt
        assert "Gemini prompting strategy" in prompt

    def test_xai_prompt_structure(self):
        """Test that xAI prompt has correct structure."""
        prompt = XAI_OPTIMIZATION_PROMPT.get_prompt(
            original_prompt="Test prompt", target_model="grok-4-1-fast-reasoning"
        )

        assert "xAI Grok text models" in prompt
        assert "grok-4-1-fast-reasoning" in prompt
        assert "Test prompt" in prompt
        assert "Return ONLY the optimized prompt" in prompt
        assert "xAI-specific prompting guidance to apply" in prompt

    def test_openai_description(self):
        """Test that OpenAI description is correct."""
        description = OPENAI_OPTIMIZATION_PROMPT.get_description()

        assert isinstance(description, str)
        assert len(description) > 0
        assert "OpenAI" in description

    def test_anthropic_description(self):
        """Test that Anthropic description is correct."""
        description = ANTHROPIC_OPTIMIZATION_PROMPT.get_description()

        assert isinstance(description, str)
        assert len(description) > 0
        assert "Anthropic" in description

    def test_google_description(self):
        """Test that Google description is correct."""
        description = GOOGLE_OPTIMIZATION_PROMPT.get_description()

        assert isinstance(description, str)
        assert len(description) > 0
        assert "Google" in description

    def test_xai_description(self):
        """Test that xAI description is correct."""
        description = XAI_OPTIMIZATION_PROMPT.get_description()

        assert isinstance(description, str)
        assert len(description) > 0
        assert "xAI" in description


class TestGlobalPromptFunctions:
    """Test cases for global prompt functions."""

    def test_get_optimization_prompt_success(self):
        """Test getting optimization prompt successfully."""
        result = get_optimization_prompt("openai", "Test prompt", "gpt-4o")

        assert isinstance(result, str)
        assert len(result) > 0
        assert "OpenAI GPT models" in result
        assert "Test prompt" in result
        assert "gpt-4o" in result

    def test_get_optimization_prompt_invalid_family(self):
        """Test getting optimization prompt with invalid family."""
        with pytest.raises(
            ValueError,
            match="No optimization prompt registered for family: invalid_family",
        ):
            get_optimization_prompt("invalid_family", "Test prompt", "gpt-4o")

    def test_get_optimization_description_success(self):
        """Test getting optimization description successfully."""
        result = get_optimization_description("openai")

        assert isinstance(result, str)
        assert len(result) > 0
        assert "OpenAI" in result

    def test_get_optimization_description_invalid_family(self):
        """Test getting optimization description with invalid family."""
        with pytest.raises(
            ValueError,
            match="No optimization prompt registered for family: invalid_family",
        ):
            get_optimization_description("invalid_family")

    def test_get_supported_families(self):
        """Test getting supported families."""
        result = get_supported_families()

        assert isinstance(result, list)
        assert len(result) > 0
        assert "openai" in result
        assert "anthropic" in result
        assert "google" in result
        assert "xai" in result


class TestPromptIntegration:
    """Test cases for prompt integration and consistency."""

    def test_all_families_have_prompts(self):
        """Test that all supported families have prompts registered."""
        families = get_supported_families()

        for family in families:
            prompt = get_optimization_prompt(family, "Test prompt", "test-model")
            description = get_optimization_description(family)

            assert isinstance(prompt, str)
            assert len(prompt) > 0
            assert isinstance(description, str)
            assert len(description) > 0

    def test_prompt_consistency(self):
        """Test that prompts are consistent across families."""
        test_prompt = "Test prompt"
        test_model = "test-model"

        families = get_supported_families()

        for family in families:
            prompt = get_optimization_prompt(family, test_prompt, test_model)

            # All prompts should contain the original prompt and target model
            assert test_prompt in prompt
            assert test_model in prompt

            # All prompts should instruct to return only the optimized prompt
            assert "Return ONLY the optimized prompt" in prompt

    def test_description_consistency(self):
        """Test that descriptions are consistent across families."""
        families = get_supported_families()

        for family in families:
            description = get_optimization_description(family)

            # All descriptions should be strings with content
            assert isinstance(description, str)
            assert len(description) > 0

            # All descriptions should mention the family name
            assert family in description.lower()

    def test_prompt_manager_global_instance(self):
        """Test that the global prompt manager instance works correctly."""
        # Test that the global functions use the same manager instance
        prompt1 = get_optimization_prompt("openai", "Test prompt", "gpt-4o")
        prompt2 = get_optimization_prompt("openai", "Test prompt", "gpt-4o")

        assert prompt1 == prompt2  # Should be the same result

        # Test that the manager has all families registered
        families = get_supported_families()
        assert len(families) >= 4  # Should include OpenAI, Anthropic, Google, and xAI

    def test_prompt_parameter_substitution(self):
        """Test that prompt parameters are properly substituted."""
        original_prompt = "Original test prompt with {variables}"
        target_model = "gpt-4o"

        prompt = get_optimization_prompt("openai", original_prompt, target_model)

        # The original prompt should be included in the system prompt
        assert original_prompt in prompt
        assert target_model in prompt

        # The system prompt should be properly formatted
        assert "You are an expert at optimizing prompts" in prompt



"""
Base optimization prompt template and utilities.

This module provides the foundation for family-specific optimization prompts
using Jinja2 templating for centralized prompt management.
"""

from abc import ABC, abstractmethod
from pathlib import Path

from jinja2 import Environment, FileSystemLoader, Template


class TemplateManager:
    """Manages Jinja2 templates for prompt optimization."""

    def __init__(self):
        template_dir = Path(__file__).parent / "templates"
        self.env = Environment(
            loader=FileSystemLoader(template_dir),
            trim_blocks=True,
            lstrip_blocks=True,
        )

    def get_template(self, template_name: str) -> Template:
        """Get a Jinja template by name."""
        return self.env.get_template(f"{template_name}.jinja")

    def render_template(self, template_name: str, **kwargs) -> str:
        """Render a template with the given context variables."""
        template = self.get_template(template_name)
        return template.render(**kwargs)


# Global template manager instance
_template_manager = TemplateManager()


class BaseOptimizationPrompt(ABC):
    """Abstract base class for optimization prompts using Jinja templates."""

    def __init__(self, template_name: str):
        self.template_name = template_name
        self.template_manager = _template_manager

    def get_prompt(self, original_prompt: str, target_model: str) -> str:
        """
        Get the optimization prompt for a specific model family.

        Args:
            original_prompt: The original prompt to optimize
            target_model: The target model name

        Returns:
            The optimization prompt rendered from Jinja template
        """
        return self.template_manager.render_template(
            self.template_name,
            original_prompt=original_prompt,
            target_model=target_model,
            model_family=self.get_model_family(),
        )

    @abstractmethod
    def get_model_family(self) -> str:
        """Get the model family name for this optimization prompt."""

    @abstractmethod
    def get_description(self) -> str:
        """
        Get a description of this optimization approach.

        Returns:
            Description of the optimization strategy
        """


class OptimizationPromptManager:
    """Manager for optimization prompts across different model families."""

    def __init__(self):
        self._prompts: dict[str, BaseOptimizationPrompt] = {}

    def register_prompt(self, family: str, prompt: BaseOptimizationPrompt) -> None:
        """Register an optimization prompt for a model family."""
        self._prompts[family] = prompt

    def get_prompt(self, family: str, original_prompt: str, target_model: str) -> str:
        """Get the optimization prompt for a specific family."""
        if family not in self._prompts:
            raise ValueError(f"No optimization prompt registered for family: {family}")

        return self._prompts[family].get_prompt(original_prompt, target_model)

    def get_description(self, family: str) -> str:
        """Get the description for a specific family."""
        if family not in self._prompts:
            raise ValueError(f"No optimization prompt registered for family: {family}")

        return self._prompts[family].get_description()

    def get_supported_families(self) -> list:
        """Get list of supported model families."""
        return list(self._prompts.keys())



"""
Provider factory for creating and managing provider clients.

This module implements the factory pattern for creating provider-specific
clients and managing their lifecycle.
"""

import asyncio
from typing import Any

from ..clients.base import BaseClient
from ..exceptions import PAGANSConfigurationError
from ..models import Provider


class ProviderFactory:
    """
    Factory for creating and managing provider clients.

    This class implements the factory pattern to create provider-specific
    clients and manage their configuration and lifecycle.
    """

    def __init__(self):
        """Initialize the provider factory."""
        self._clients: dict[Provider, BaseClient] = {}
        self._client_classes: dict[Provider, type[BaseClient]] = {}
        self._configs: dict[Provider, dict[str, Any]] = {}

    def register_client(
        self,
        provider: Provider,
        client_class: type[BaseClient],
        config: dict[str, Any] | None = None,
    ) -> None:
        """
        Register a client class for a provider.

        Args:
            provider: The provider to register the client for
            client_class: The client class to register
            config: Default configuration for the provider
        """
        self._client_classes[provider] = client_class
        if config:
            self._configs[provider] = config

    def get_client(
        self, provider: Provider, config: dict[str, Any] | None = None
    ) -> BaseClient:
        """
        Get or create a client for the specified provider.

        Args:
            provider: The provider to get the client for
            config: Configuration to use for the client

        Returns:
            The client instance for the provider

        Raises:
            ConfigurationError: If no client is registered for the provider
        """
        if provider not in self._client_classes:
            raise PAGANSConfigurationError(
                f"No client registered for provider: {provider.value}"
            )

        if provider not in self._clients:
            # Merge default config with provided config
            client_config = self._configs.get(provider, {}).copy()
            if config:
                client_config.update(config)

            # Create new client instance
            client_class = self._client_classes[provider]
            self._clients[provider] = client_class(provider, client_config)

        return self._clients[provider]

    def get_or_create_client(
        self, provider: Provider, config: dict[str, Any] | None = None
    ) -> BaseClient:
        """
        Get an existing client or create a new one for the provider.

        Args:
            provider: The provider to get/create the client for
            config: Configuration to use for the client

        Returns:
            The client instance for the provider
        """
        return self.get_client(provider, config)

    def close_client(self, provider: Provider) -> None:
        """
        Close and remove a client for the specified provider.

        Args:
            provider: The provider whose client should be closed
        """
        if provider in self._clients:
            asyncio.create_task(self._clients[provider].close())
            del self._clients[provider]

    def close_all_clients(self) -> None:
        """Close all registered clients."""
        for provider in list(self._clients.keys()):
            self.close_client(provider)

    def is_client_registered(self, provider: Provider) -> bool:
        """
        Check if a client is registered for the provider.

        Args:
            provider: The provider to check

        Returns:
            True if a client is registered, False otherwise
        """
        return provider in self._client_classes

    def get_registered_providers(self) -> list[Provider]:
        """
        Get all providers that have registered clients.

        Returns:
            List of registered providers
        """
        return list(self._client_classes.keys())

    def get_active_clients(self) -> dict[Provider, BaseClient]:
        """
        Get all currently active clients.

        Returns:
            Dictionary mapping providers to their active clients
        """
        return self._clients.copy()


# Global factory instance
_provider_factory = ProviderFactory()


def get_provider_factory() -> ProviderFactory:
    """
    Get the global provider factory instance.

    Returns:
        The global ProviderFactory instance
    """
    return _provider_factory


def register_provider_client(
    provider: Provider,
    client_class: type[BaseClient],
    config: dict[str, Any] | None = None,
) -> None:
    """
    Register a client class for a provider in the global factory.

    Args:
        provider: The provider to register the client for
        client_class: The client class to register
        config: Default configuration for the provider
    """
    get_provider_factory().register_client(provider, client_class, config)


def get_provider_client(
    provider: Provider, config: dict[str, Any] | None = None
) -> BaseClient:
    """
    Get or create a client for the specified provider using the global factory.

    Args:
        provider: The provider to get the client for
        config: Configuration to use for the client

    Returns:
        The client instance for the provider
    """
    return get_provider_factory().get_client(provider, config)



"""
xAI family optimization prompts.

This module contains prompt optimization guidance for xAI Grok text models.
"""

from .base import BaseOptimizationPrompt


class XAIOptimizationPrompt(BaseOptimizationPrompt):
    """Optimization prompt for xAI Grok models."""

    def __init__(self):
        super().__init__("xai")

    def get_model_family(self) -> str:
        """Get the model family name."""
        return "xAI Grok"

    def get_prompt(self, original_prompt: str, target_model: str) -> str:
        """Get the xAI-specific optimization prompt."""
        return super().get_prompt(original_prompt, target_model)

    def get_description(self) -> str:
        """Get description of xAI optimization approach."""
        return """
        Optimization strategy for xAI Grok text models focusing on:
        - Strong context grounding with relevant project or task constraints
        - Explicit, concrete goals and completion criteria
        - Detailed system instructions for behavior, edge cases, and quality bar
        - Iterative refinement guidance when first-pass output is insufficient
        - Structured context blocks (XML or Markdown sections) for clarity

        Based on xAI developer guidance for Grok model prompting and migration.
        """


# Global instance for easy access
XAI_OPTIMIZATION_PROMPT = XAIOptimizationPrompt()



"""
Anthropic family optimization prompts.

This module contains detailed, research-specific optimization prompts
for Anthropic's Claude models based on official documentation and best practices.
"""

from .base import BaseOptimizationPrompt


class AnthropicOptimizationPrompt(BaseOptimizationPrompt):
    """Optimization prompt for Anthropic Claude models."""

    def __init__(self):
        super().__init__("anthropic")

    def get_model_family(self) -> str:
        """Get the model family name."""
        return "Anthropic Claude"

    def get_prompt(self, original_prompt: str, target_model: str) -> str:
        """
        Get the Anthropic-specific optimization prompt.

        Based on Anthropic's prompt engineering best practices and documentation.
        """
        return super().get_prompt(original_prompt, target_model)

    def get_description(self) -> str:
        """Get description of Anthropic optimization approach."""
        return """
        Optimization strategy for Anthropic Claude text models focusing on:
        - clear and direct instructions
        - context and motivation for better task targeting
        - XML-tag structure for complex prompts
        - few-shot examples for consistency and steerability
        - explicit output format constraints

        Based on Anthropic Claude prompt engineering best practices for Claude 4.x models.
        """


# Global instance for easy access
ANTHROPIC_OPTIMIZATION_PROMPT = AnthropicOptimizationPrompt()



"""
Constants and configuration values for PAGANS.

This module contains all the default values, environment variable names,
and other constants used throughout the package.
"""

# Environment variable names
ENV_OPENROUTER_API_KEY = "OPENROUTER_API_KEY"
ENV_OPENROUTER_BASE_URL = "OPENROUTER_BASE_URL"
ENV_OPENAI_API_KEY = "OPENAI_API_KEY"
ENV_OPENAI_BASE_URL = "OPENAI_BASE_URL"
ENV_GOOGLE_API_KEY = "GOOGLE_API_KEY"
ENV_GOOGLE_BASE_URL = "GOOGLE_BASE_URL"
ENV_ANTHROPIC_API_KEY = "ANTHROPIC_API_KEY"
ENV_ANTHROPIC_BASE_URL = "ANTHROPIC_BASE_URL"
ENV_ZAI_API_KEY = "ZAI_API_KEY"
ENV_ZAI_BASE_URL = "ZAI_BASE_URL"
ENV_PAGANS_OPTIMIZER_MODEL = "PAGANS_OPTIMIZER_MODEL"

# Default configuration values
DEFAULT_PAGANS_OPTIMIZER_MODEL = "openai/gpt-4o-mini"
DEFAULT_BASE_URL = "https://openrouter.ai/api/v1"
DEFAULT_OPENAI_BASE_URL = "https://api.openai.com/v1"
DEFAULT_GOOGLE_BASE_URL = "https://generativelanguage.googleapis.com/v1beta/openai"
DEFAULT_ANTHROPIC_BASE_URL = "https://api.anthropic.com/v1"
DEFAULT_ZAI_BASE_URL = "https://api.z.ai/api/paas/v4"
DEFAULT_TIMEOUT = 30.0  # seconds
DEFAULT_MAX_RETRIES = 3
DEFAULT_RETRY_DELAY = 1.0  # seconds

# API configuration
API_VERSION = "v1"
CHAT_COMPLETIONS_ENDPOINT = "/chat/completions"

# HTTP headers
DEFAULT_HEADERS = {
    "Content-Type": "application/json",
    "Accept": "application/json",
}

# Model family names
FAMILY_OPENAI = "openai"
FAMILY_ANTHROPIC = "anthropic"
FAMILY_GOOGLE = "google"
FAMILY_XAI = "xai"

# Error messages
ERROR_MESSAGES = {
    "missing_api_key": "OpenRouter API key is required. Set OPENROUTER_API_KEY environment variable.",
    "invalid_model": "Invalid model name: {model_name}",
    "network_error": "Network error occurred while optimizing prompt",
    "api_error": "OpenRouter API error: {error}",
    "timeout_error": "Request timed out after {timeout} seconds",
    "rate_limit_error": "Rate limit exceeded. Please wait before trying again.",
    "unknown_error": "Unknown error occurred: {error}",
}

# Performance targets
PAGANS_TARGET_OPTIMIZATION_TIME = 10.0  # seconds
PAGANS_MAX_TOKENS = 8000  # Maximum tokens for optimization prompts



"""
Optimization prompts for different LLM model families.

This module contains family-specific optimization prompts tailored to each
model architecture based on official documentation and best practices.
"""

from .anthropic import ANTHROPIC_OPTIMIZATION_PROMPT
from .base import BaseOptimizationPrompt, OptimizationPromptManager
from .google import GOOGLE_OPTIMIZATION_PROMPT
from .openai import OPENAI_OPTIMIZATION_PROMPT
from .xai import XAI_OPTIMIZATION_PROMPT

_prompt_manager = OptimizationPromptManager()

_prompt_manager.register_prompt("openai", OPENAI_OPTIMIZATION_PROMPT)
_prompt_manager.register_prompt("anthropic", ANTHROPIC_OPTIMIZATION_PROMPT)
_prompt_manager.register_prompt("google", GOOGLE_OPTIMIZATION_PROMPT)
_prompt_manager.register_prompt("xai", XAI_OPTIMIZATION_PROMPT)


def get_optimization_prompt(
    family: str, original_prompt: str, target_model: str
) -> str:
    """
    Get the optimization prompt for a specific model family.

    Args:
        family: The model family name (e.g., "openai", "anthropic")
        original_prompt: The original prompt to optimize
        target_model: The target model name

    Returns:
        The optimization prompt for the specified family

    Raises:
        ValueError: If the family is not supported
    """
    return _prompt_manager.get_prompt(family, original_prompt, target_model)


def get_optimization_description(family: str) -> str:
    """
    Get the description for a specific model family's optimization approach.

    Args:
        family: The model family name (e.g., "openai", "anthropic")

    Returns:
        The description of the optimization approach

    Raises:
        ValueError: If the family is not supported
    """
    return _prompt_manager.get_description(family)


def get_supported_families() -> list:
    """
    Get list of supported model families.

    Returns:
        List of supported family names
    """
    return _prompt_manager.get_supported_families()


__all__ = [
    "ANTHROPIC_OPTIMIZATION_PROMPT",
    "GOOGLE_OPTIMIZATION_PROMPT",
    "OPENAI_OPTIMIZATION_PROMPT",
    "XAI_OPTIMIZATION_PROMPT",
    "BaseOptimizationPrompt",
    "OptimizationPromptManager",
    "get_optimization_description",
    "get_optimization_prompt",
    "get_supported_families",
]



"""
Edge case tests for PAGANS.

This module contains tests for edge cases, boundary conditions,
and unusual scenarios that might cause issues.
"""

import asyncio
from unittest.mock import AsyncMock, patch

import pytest

from src.pagans.core import PAGANSOptimizer
from src.pagans.exceptions import (
    PAGANSError,
    PAGANSNetworkError,
)
from src.pagans.models import ModelFamily, OptimizationResult, Provider


class TestExtremeInputHandling:
    """Test cases for extreme and unusual input scenarios."""

    @pytest.fixture
    def mock_client(self):
        """Create a mock client for testing."""
        with patch("src.pagans.clients.OpenRouterClient") as mock_client_class:
            mock_client = AsyncMock()
            mock_client_class.return_value = mock_client

            mock_client.optimize_prompt.return_value = "Optimized"
            mock_client.validate_model.return_value = True

            yield mock_client

    def test_empty_prompt_handling(self, mock_client):
        """Test handling of empty prompts."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            with pytest.raises(ValueError, match="Prompt cannot be empty"):
                asyncio.run(optimizer.optimize(prompt="", target_model="openai/gpt-4o"))

    def test_whitespace_only_prompt(self, mock_client):
        """Test handling of whitespace-only prompts."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            with pytest.raises(ValueError, match="Prompt cannot be empty"):
                asyncio.run(
                    optimizer.optimize(prompt="   \n\t  ", target_model="openai/gpt-4o")
                )

    def test_very_long_prompt(self, mock_client):
        """Test handling of extremely long prompts."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            # Create a very long prompt (100KB+)
            long_prompt = (
                "Write a comprehensive tutorial about Python programming. " * 2000
            )

            result = asyncio.run(
                optimizer.optimize(prompt=long_prompt, target_model="openai/gpt-4o")
            )

            assert isinstance(result, OptimizationResult)
            assert len(result.original) > 100000  # Ensure it's actually long
            assert result.optimized == "Optimized"

    def test_unicode_and_special_characters(self, mock_client):
        """Test handling of Unicode and special characters."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            unicode_prompt = "Write a Python function using emojis 😀 and special chars: àáâãäå, 中文, русский"

            result = asyncio.run(
                optimizer.optimize(prompt=unicode_prompt, target_model="openai/gpt-4o")
            )

            assert isinstance(result, OptimizationResult)
            assert result.original == unicode_prompt
            assert result.optimized == "Optimized"

    def test_binary_data_in_prompt(self, mock_client):
        """Test handling of binary-like data in prompts."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            # Create a prompt with binary-like content
            binary_prompt = "Analyze this data: " + "".join(
                chr(i) for i in range(32, 127)
            )

            result = asyncio.run(
                optimizer.optimize(prompt=binary_prompt, target_model="openai/gpt-4o")
            )

            assert isinstance(result, OptimizationResult)
            assert result.optimized == "Optimized"

    def test_extremely_short_prompt(self, mock_client):
        """Test handling of very short prompts."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            result = asyncio.run(
                optimizer.optimize(prompt="Hi", target_model="openai/gpt-4o")
            )

            assert isinstance(result, OptimizationResult)
            assert result.original == "Hi"
            assert result.optimized == "Optimized"


class TestModelEdgeCases:
    """Test cases for model-related edge cases."""

    @pytest.fixture
    def mock_client(self):
        """Create a mock client for testing."""
        with patch("src.pagans.clients.OpenRouterClient") as mock_client_class:
            mock_client = AsyncMock()
            mock_client_class.return_value = mock_client

            yield mock_client

    def test_unknown_model_family(self, mock_client):
        """Test handling of unknown model families."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            from src.pagans.exceptions import PAGANSModelNotFoundError

            optimizer = PAGANSOptimizer(api_key="test-key")

            with pytest.raises(PAGANSModelNotFoundError):
                asyncio.run(
                    optimizer.optimize(
                        prompt="Test prompt", target_model="unknown/model-v123"
                    )
                )

    def test_model_with_special_characters(self, mock_client):
        """Test handling of model names with special characters."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            from src.pagans.exceptions import PAGANSModelNotFoundError

            optimizer = PAGANSOptimizer(api_key="test-key")

            special_models = [
                "model/with/slashes",
                "model-with-dashes",
                "model.with.dots",
                "model_underscore",
                "model@symbol",
            ]

            for model in special_models:
                with pytest.raises(PAGANSModelNotFoundError):
                    asyncio.run(
                        optimizer.optimize(prompt="Test prompt", target_model=model)
                    )

    def test_case_sensitivity_in_model_names(self, mock_client):
        """Test case sensitivity handling in model names."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            mock_client.optimize_prompt.return_value = "Optimized"
            mock_client.validate_model.return_value = True

            result = asyncio.run(
                optimizer.optimize(
                    prompt="Test prompt",
                    target_model="OpenAI/GPT-4O",  # Mixed case
                )
            )

            assert isinstance(result, OptimizationResult)
            assert result.target_model == "OpenAI/GPT-4O"


class TestConfigurationEdgeCases:
    """Test cases for configuration edge cases."""

    def test_extreme_timeout_values(self):
        """Test handling of extreme timeout values."""
        with patch("src.pagans.core.OpenRouterClient") as mock_client_class:
            # Very short timeout
            optimizer1 = PAGANSOptimizer(api_key="test-key", timeout=0.001)
            mock_client_class.assert_called_with(
                provider=Provider.OPENROUTER,
                config={
                    "api_key": "test-key",
                    "base_url": "https://openrouter.ai/api/v1",
                    "timeout": 0.001,
                    "max_retries": 3,
                    "retry_delay": 1.0,
                },
            )

            # Very long timeout
            optimizer2 = PAGANSOptimizer(api_key="test-key", timeout=3600.0)
            mock_client_class.assert_called_with(
                provider=Provider.OPENROUTER,
                config={
                    "api_key": "test-key",
                    "base_url": "https://openrouter.ai/api/v1",
                    "timeout": 3600.0,
                    "max_retries": 3,
                    "retry_delay": 1.0,
                },
            )

            # Zero timeout (should handle gracefully)
            optimizer3 = PAGANSOptimizer(api_key="test-key", timeout=0.0)
            mock_client_class.assert_called_with(
                provider=Provider.OPENROUTER,
                config={
                    "api_key": "test-key",
                    "base_url": "https://openrouter.ai/api/v1",
                    "timeout": 0.0,
                    "max_retries": 3,
                    "retry_delay": 1.0,
                },
            )

    def test_extreme_retry_values(self):
        """Test handling of extreme retry values."""
        with patch("src.pagans.core.OpenRouterClient") as mock_client_class:
            # Zero retries
            optimizer1 = PAGANSOptimizer(api_key="test-key", max_retries=0)
            mock_client_class.assert_called_with(
                provider=Provider.OPENROUTER,
                config={
                    "api_key": "test-key",
                    "base_url": "https://openrouter.ai/api/v1",
                    "timeout": 30.0,
                    "max_retries": 0,
                    "retry_delay": 1.0,
                },
            )

            # Very high retry count
            optimizer2 = PAGANSOptimizer(api_key="test-key", max_retries=100)
            mock_client_class.assert_called_with(
                provider=Provider.OPENROUTER,
                config={
                    "api_key": "test-key",
                    "base_url": "https://openrouter.ai/api/v1",
                    "timeout": 30.0,
                    "max_retries": 100,
                    "retry_delay": 1.0,
                },
            )

    def test_extreme_concurrency_values(self):
        """Test handling of extreme concurrency values."""
        with patch("src.pagans.core.OpenRouterClient") as mock_client_class:
            with patch("httpx.AsyncClient") as mock_httpx_class:
                mock_client = AsyncMock()
                mock_client_class.return_value = mock_client
                mock_client.optimize_prompt.return_value = "Optimized"
                mock_client.validate_model.return_value = True

                # Mock the httpx client
                mock_httpx = AsyncMock()
                mock_httpx_class.return_value = mock_httpx
                mock_httpx.request.return_value = AsyncMock()
                mock_httpx.request.return_value.status_code = 200
                mock_httpx.request.return_value.json.return_value = {
                    "choices": [{"message": {"content": "Optimized"}}]
                }

                optimizer = PAGANSOptimizer(api_key="test-key")

                # Test with very high concurrency
                prompts = ["Test"] * 10
                results = asyncio.run(
                    optimizer.optimize_multiple(
                        prompts=prompts,
                        target_model="openai/gpt-4o",
                        max_concurrent=100,  # Very high concurrency
                    )
                )

                assert len(results) == 10

    def test_malformed_urls(self):
        """Test handling of malformed URLs."""
        # These should not raise exceptions during initialization
        configs = [
            {"base_url": "not-a-url"},
            {"base_url": "http://"},
            {"base_url": "https://"},
            {"base_url": "ftp://example.com"},
            {"base_url": ""},
        ]

        for config in configs:
            try:
                optimizer = PAGANSOptimizer(api_key="test-key", **config)
                # Should not crash during initialization
                assert optimizer is not None
            except Exception:
                # Some malformed URLs might raise exceptions, which is acceptable
                pass


class TestBatchProcessingEdgeCases:
    """Test cases for batch processing edge cases."""

    @pytest.fixture
    def mock_client(self):
        """Create a mock client for testing."""
        with patch("src.pagans.clients.OpenRouterClient") as mock_client_class:
            mock_client = AsyncMock()
            mock_client_class.return_value = mock_client

            mock_client.optimize_prompt.return_value = "Optimized"
            mock_client.validate_model.return_value = True

            yield mock_client

    def test_empty_batch(self, mock_client):
        """Test handling of empty batch."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            results = asyncio.run(
                optimizer.optimize_multiple(prompts=[], target_model="openai/gpt-4o")
            )

            assert results == []

    def test_single_item_batch(self, mock_client):
        """Test batch processing with single item."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            results = asyncio.run(
                optimizer.optimize_multiple(
                    prompts=["Single prompt"], target_model="openai/gpt-4o"
                )
            )

            assert len(results) == 1
            assert isinstance(results[0], OptimizationResult)
            assert results[0].original == "Single prompt"

    def test_mixed_prompt_lengths_in_batch(self, mock_client):
        """Test batch with prompts of vastly different lengths."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            prompts = [
                "Short",
                "A" * 1000,  # Medium
                "B" * 10000,  # Long
                "C" * 50000,  # Very long
            ]

            results = asyncio.run(
                optimizer.optimize_multiple(
                    prompts=prompts, target_model="openai/gpt-4o"
                )
            )

            assert len(results) == len(prompts)
            for i, result in enumerate(results):
                assert isinstance(result, OptimizationResult)
                assert len(result.original) == len(prompts[i])

    def test_batch_with_duplicate_prompts(self, mock_client):
        """Test batch processing with duplicate prompts."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            prompts = ["Same prompt"] * 5

            results = asyncio.run(
                optimizer.optimize_multiple(
                    prompts=prompts, target_model="openai/gpt-4o"
                )
            )

            assert len(results) == 5
            for result in results:
                assert isinstance(result, OptimizationResult)
                assert result.original == "Same prompt"
                assert result.optimized == "Optimized"


class TestErrorRecoveryEdgeCases:
    """Test cases for error recovery in edge scenarios."""

    @pytest.fixture
    def failing_client(self):
        """Create a mock client that can simulate various failures."""
        with patch("src.pagans.clients.OpenRouterClient") as mock_client_class:
            mock_client = AsyncMock()
            mock_client_class.return_value = mock_client

            yield mock_client

    def test_partial_batch_failure_recovery(self, failing_client):
        """Test recovery when some items in a batch fail."""
        with patch("src.pagans.core.OpenRouterClient", return_value=failing_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            # Mock partial failures in batch
            call_count = 0

            async def partial_failure_optimize(*args, **kwargs):
                nonlocal call_count
                call_count += 1

                if call_count % 3 == 0:  # Every third call fails
                    raise PAGANSNetworkError("Intermittent failure")

                return f"Optimized {call_count}"

            failing_client.optimize_prompt = partial_failure_optimize
            failing_client.validate_model.return_value = True

            prompts = ["Prompt"] * 9  # 9 prompts, 3 should fail

            # This should handle partial failures gracefully
            results = asyncio.run(
                optimizer.optimize_multiple(
                    prompts=prompts, target_model="openai/gpt-4o"
                )
            )

            # Should have some successful results
            successful_results = [
                r for r in results if isinstance(r, OptimizationResult)
            ]
            assert len(successful_results) > 0

    def test_network_partition_simulation(self, failing_client):
        """Test behavior during simulated network partitions."""
        with patch("src.pagans.core.OpenRouterClient", return_value=failing_client):
            optimizer = PAGANSOptimizer(api_key="test-key", max_retries=2)

            # Simulate complete network failure
            async def network_failure(*args, **kwargs):
                raise PAGANSNetworkError("Network partition")

            failing_client.optimize_prompt = network_failure
            failing_client.validate_model.return_value = True

            with pytest.raises(PAGANSError):
                asyncio.run(
                    optimizer.optimize(
                        prompt="Test prompt", target_model="openai/gpt-4o"
                    )
                )

    def test_extreme_memory_pressure(self, failing_client):
        """Test behavior under extreme memory pressure."""
        with patch("src.pagans.core.OpenRouterClient", return_value=failing_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            # Create extremely large prompts that might cause memory issues
            huge_prompts = ["X" * 1000000] * 10  # 10 prompts of 1MB each

            failing_client.optimize_prompt.return_value = "Optimized"

            results = asyncio.run(
                optimizer.optimize_multiple(
                    prompts=huge_prompts,
                    target_model="openai/gpt-4o",
                    max_concurrent=2,  # Low concurrency to manage memory
                )
            )

            assert len(results) == len(huge_prompts)

    def test_rapid_consecutive_requests(self, failing_client):
        """Test handling of rapid consecutive requests."""
        with patch("src.pagans.core.OpenRouterClient", return_value=failing_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            failing_client.optimize_prompt.return_value = "Optimized"

            # Send many requests in rapid succession
            async def rapid_requests():
                tasks = []
                for i in range(50):
                    task = optimizer.optimize(
                        prompt=f"Rapid request {i}", target_model="openai/gpt-4o"
                    )
                    tasks.append(task)

                results = await asyncio.gather(*tasks)
                return results

            results = asyncio.run(rapid_requests())

            assert len(results) == 50
            for result in results:
                assert isinstance(result, OptimizationResult)


class TestDataIntegrity:
    """Test cases for data integrity and consistency."""

    @pytest.fixture
    def mock_client(self):
        """Create a mock client for testing."""
        with patch("src.pagans.clients.OpenRouterClient") as mock_client_class:
            mock_client = AsyncMock()
            mock_client_class.return_value = mock_client

            yield mock_client

    def test_prompt_content_preservation(self, mock_client):
        """Test that prompt content is preserved exactly."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            test_prompts = [
                "Simple prompt",
                "Prompt with\nnewlines",
                "Prompt with\ttabs",
                "Prompt with unicode: 🎯🚀💡",
                "Prompt with quotes: \"single\" and 'double'",
                "Prompt with backslashes: \\ and \\\\",
            ]

            mock_client.optimize_prompt.return_value = "Optimized"

            for original_prompt in test_prompts:
                result = asyncio.run(
                    optimizer.optimize(
                        prompt=original_prompt, target_model="openai/gpt-4o"
                    )
                )

                assert result.original == original_prompt

    def test_metadata_consistency(self, mock_client):
        """Test that metadata is consistent across operations."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            mock_client.optimize_prompt.return_value = "Optimized"

            result = asyncio.run(
                optimizer.optimize(
                    prompt="Test prompt",
                    target_model="openai/gpt-4o",
                    optimization_notes="Test notes",
                )
            )

            assert result.target_model == "openai/gpt-4o"
            assert result.target_family == ModelFamily.OPENAI
            assert result.optimization_notes == "Test notes"

    def test_result_object_immutability(self, mock_client):
        """Test that result objects are properly immutable."""
        with patch("src.pagans.core.OpenRouterClient", return_value=mock_client):
            optimizer = PAGANSOptimizer(api_key="test-key")

            mock_client.optimize_prompt.return_value = "Optimized"

            result = asyncio.run(
                optimizer.optimize(prompt="Test prompt", target_model="openai/gpt-4o")
            )

            # Attempt to modify result
            result.optimized = "Modified"
            result.original = "Modified original"

            # Verify values were changed (object is mutable)
            assert result.optimized == "Modified"
            assert result.original == "Modified original"



"""
Base client abstract class for PAGANS provider clients.

This module defines the abstract base class that all provider-specific
clients must implement to ensure consistent interface across providers.
"""

from abc import ABC, abstractmethod
from typing import Any

from ..models import Provider


class BaseClient(ABC):
    """
    Abstract base class for all provider clients.

    This class defines the interface that all provider-specific clients
    must implement to ensure consistency across different providers.
    """

    def __init__(self, provider: Provider, config: dict[str, Any]):
        """
        Initialize the base client.

        Args:
            provider: The provider this client handles
            config: Configuration dictionary for the provider
        """
        self.provider = provider
        self.config = config

    @abstractmethod
    async def optimize_prompt(
        self,
        prompt: str,
        model: str,
        system_prompt: str,
        temperature: float = 0.1,
        max_tokens: int = 2000,
    ) -> str:
        """
        Optimize a prompt using the provider's API.

        Args:
            prompt: The original prompt to optimize
            model: The model to use for optimization
            system_prompt: The system prompt for optimization
            temperature: Temperature for response generation
            max_tokens: Maximum tokens for the response

        Returns:
            The optimized prompt

        Raises:
            Various exceptions based on provider-specific errors
        """

    @abstractmethod
    async def validate_model(self, model_name: str) -> bool:
        """
        Validate if a model is available on this provider.

        Args:
            model_name: Name of the model to validate

        Returns:
            True if model is available, False otherwise
        """

    @abstractmethod
    async def get_models(self) -> dict[str, Any]:
        """
        Get available models from the provider.

        Returns:
            Dictionary of available models

        Raises:
            Various exceptions based on provider-specific errors
        """

    @abstractmethod
    async def close(self) -> None:
        """Close the client and clean up resources."""

    @abstractmethod
    async def __aenter__(self):
        """Async context manager entry."""
        return self

    @abstractmethod
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """Async context manager exit."""
        await self.close()



"""
Model family registry for extensible model support.

This module provides a registry system for managing model families,
providers, and their relationships in an extensible way.
"""

from dataclasses import dataclass

from . import ModelFamily, Provider


@dataclass
class ModelFamilyInfo:
    """Information about a model family."""

    family: ModelFamily
    display_name: str
    description: str
    supported_providers: list[Provider]
    default_models: dict[str, str]  # short_name -> provider_model_name
    aliases: dict[str, str]  # alias -> canonical_name


@dataclass
class ProviderInfo:
    """Information about a provider."""

    provider: Provider
    display_name: str
    description: str
    base_url: str
    api_version: str
    supported_families: list[ModelFamily]
    model_mappings: dict[str, str]  # short_name -> provider_model_name
    auth_header: str
    auth_prefix: str = "Bearer"


class ModelRegistry:
    """
    Registry for managing model families and providers.

    This class provides an extensible system for registering new model
    families and providers, and managing their relationships.
    """

    def __init__(self):
        """Initialize the model registry."""
        self._families: dict[ModelFamily, ModelFamilyInfo] = {}
        self._providers: dict[Provider, ProviderInfo] = {}
        self._model_to_family: dict[str, ModelFamily] = {}
        self._family_to_models: dict[ModelFamily, set[str]] = {}

    def register_family(self, family_info: ModelFamilyInfo) -> None:
        """
        Register a model family.

        Args:
            family_info: Information about the model family
        """
        self._families[family_info.family] = family_info

        # Update model mappings
        for short_name, provider_model in family_info.default_models.items():
            self._model_to_family[short_name] = family_info.family
            self._model_to_family[provider_model] = family_info.family

        # Update family to models mapping
        if family_info.family not in self._family_to_models:
            self._family_to_models[family_info.family] = set()
        self._family_to_models[family_info.family].update(
            family_info.default_models.keys()
        )
        self._family_to_models[family_info.family].update(
            family_info.default_models.values()
        )

    def register_provider(self, provider_info: ProviderInfo) -> None:
        """
        Register a provider.

        Args:
            provider_info: Information about the provider
        """
        self._providers[provider_info.provider] = provider_info

    def get_family_info(self, family: ModelFamily) -> ModelFamilyInfo | None:
        """
        Get information about a model family.

        Args:
            family: The model family to get information for

        Returns:
            ModelFamilyInfo if found, None otherwise
        """
        return self._families.get(family)

    def get_provider_info(self, provider: Provider) -> ProviderInfo | None:
        """
        Get information about a provider.

        Args:
            provider: The provider to get information for

        Returns:
            ProviderInfo if found, None otherwise
        """
        return self._providers.get(provider)

    def get_model_family(self, model_name: str) -> ModelFamily | None:
        """
        Get the model family for a model name.

        Args:
            model_name: The model name to look up

        Returns:
            ModelFamily if found, None otherwise
        """
        # Direct lookup
        if model_name in self._model_to_family:
            return self._model_to_family[model_name]

        # Try case-insensitive lookup
        model_name_lower = model_name.lower()
        for model, family in self._model_to_family.items():
            if model.lower() == model_name_lower:
                return family

        return None

    def get_models_for_family(self, family: ModelFamily) -> list[str]:
        """
        Get all models for a model family.

        Args:
            family: The model family to get models for

        Returns:
            List of model names
        """
        if family in self._family_to_models:
            return list(self._family_to_models[family])
        return []

    def get_supported_providers_for_family(self, family: ModelFamily) -> list[Provider]:
        """
        Get all providers that support a model family.

        Args:
            family: The model family to get providers for

        Returns:
            List of providers that support the family
        """
        family_info = self.get_family_info(family)
        if family_info:
            return family_info.supported_providers
        return []

    def get_provider_model_name(
        self, short_name: str, provider: Provider
    ) -> str | None:
        """
        Get the provider-specific model name for a short model name.

        Args:
            short_name: The short model name
            provider: The provider to get the model name for

        Returns:
            Provider-specific model name if found, None otherwise
        """
        provider_info = self.get_provider_info(provider)
        if provider_info and short_name in provider_info.model_mappings:
            return provider_info.model_mappings[short_name]

        # Check if family has default mapping
        family = self.get_model_family(short_name)
        if family:
            family_info = self.get_family_info(family)
            if family_info and short_name in family_info.default_models:
                return family_info.default_models[short_name]

        return None

    def is_model_supported(self, model_name: str) -> bool:
        """
        Check if a model is supported.

        Args:
            model_name: The model name to check

        Returns:
            True if the model is supported, False otherwise
        """
        return self.get_model_family(model_name) is not None

    def get_all_families(self) -> list[ModelFamily]:
        """
        Get all registered model families.

        Returns:
            List of all registered model families
        """
        return list(self._families.keys())

    def get_all_providers(self) -> list[Provider]:
        """
        Get all registered providers.

        Returns:
            List of all registered providers
        """
        return list(self._providers.keys())

    def get_families_for_provider(self, provider: Provider) -> list[ModelFamily]:
        """
        Get all model families supported by a provider.

        Args:
            provider: The provider to get families for

        Returns:
            List of model families supported by the provider
        """
        provider_info = self.get_provider_info(provider)
        if provider_info:
            return provider_info.supported_families
        return []


# Global registry instance
_model_registry = ModelRegistry()


def get_model_registry() -> ModelRegistry:
    """
    Get the global model registry instance.

    Returns:
        The global ModelRegistry instance
    """
    return _model_registry


def register_model_family(family_info: ModelFamilyInfo) -> None:
    """
    Register a model family in the global registry.

    Args:
        family_info: Information about the model family
    """
    get_model_registry().register_family(family_info)


def register_provider(provider_info: ProviderInfo) -> None:
    """
    Register a provider in the global registry.

    Args:
        provider_info: Information about the provider
    """
    get_model_registry().register_provider(provider_info)


def get_model_family(model_name: str) -> ModelFamily | None:
    """
    Get the model family for a model name using the global registry.

    Args:
        model_name: The model name to look up

    Returns:
        ModelFamily if found, None otherwise
    """
    return get_model_registry().get_model_family(model_name)


def is_model_supported(model_name: str) -> bool:
    """
    Check if a model is supported using the global registry.

    Args:
        model_name: The model name to check

    Returns:
        True if the model is supported, False otherwise
    """
    return get_model_registry().is_model_supported(model_name)



"""
Google family optimization prompts.

This module contains detailed, research-specific optimization prompts
for Google's Gemini models based on official documentation and best practices.
"""

from .base import BaseOptimizationPrompt


class GoogleOptimizationPrompt(BaseOptimizationPrompt):
    """Optimization prompt for Google Gemini models."""

    def __init__(self):
        super().__init__("google")

    def get_model_family(self) -> str:
        """Get the model family name."""
        return "Google Gemini"

    def get_prompt(self, original_prompt: str, target_model: str) -> str:
        """
        Get the Google-specific optimization prompt.

        Based on Google's prompt engineering best practices and documentation.
        """
        return super().get_prompt(original_prompt, target_model)

    def get_description(self) -> str:
        """Get description of Google optimization approach."""
        return """
        Optimization strategy for Google Gemini text models focusing on:
        - clear and specific instructions
        - explicit constraints and response format
        - structured examples for pattern-based tasks
        - system instructions for tone and verbosity control
        - iterative prompt refinement for reliability

        Based on Gemini prompt design strategies and model guidance.
        """


# Global instance for easy access
GOOGLE_OPTIMIZATION_PROMPT = GoogleOptimizationPrompt()



"""
Configuration management for different providers.

This module handles configuration loading, validation, and management
for different LLM providers.
"""

from .provider_config import (
    ConfigManager,
    ProviderConfig,
    get_config_manager,
    get_provider_config,
    set_provider_config,
)

__all__ = [
    "ConfigManager",
    "ProviderConfig",
    "get_config_manager",
    "get_provider_config",
    "set_provider_config",
]



"""
Custom exceptions for PAGANS.

This module defines the exception hierarchy used throughout the package.
"""

from typing import Any


class PAGANSError(Exception):
    """Base exception class for all PAGANS errors."""

    def __init__(self, message: str, details: Any | None = None):
        self.message = message
        self.details = details
        super().__init__(self.message)


class PAGANSOpenRouterAPIError(PAGANSError):
    """Exception raised when OpenRouter API returns an error."""

    def __init__(
        self,
        message: str,
        status_code: int | None = None,
        response_data: dict | None = None,
    ):
        super().__init__(message, response_data)
        self.status_code = status_code
        self.response_data = response_data


class PAGANSModelNotFoundError(PAGANSError):
    """Exception raised when a model is not recognized or supported."""

    def __init__(self, model_name: str):
        message = f"Model '{model_name}' is not supported or recognized"
        super().__init__(message, model_name)


class PAGANSConfigurationError(PAGANSError):
    """Exception raised when configuration is invalid or missing."""


class PAGANSNetworkError(PAGANSError):
    """Exception raised when network-related errors occur."""


class PAGANSTimeoutError(PAGANSError):
    """Exception raised when a request times out."""

    def __init__(self, timeout: float):
        message = f"Request timed out after {timeout} seconds"
        super().__init__(message, timeout)


class PAGANSRateLimitError(PAGANSError):
    """Exception raised when rate limits are exceeded."""

    def __init__(self, retry_after: int | None = None):
        message = "Rate limit exceeded"
        if retry_after:
            message += f". Please wait {retry_after} seconds before trying again."
        super().__init__(message, retry_after)


class PAGANSValidationError(PAGANSError):
    """Exception raised when input validation fails."""


class PAGANSAuthenticationError(PAGANSError):
    """Exception raised when authentication fails."""


class PAGANSQuotaExceededError(PAGANSError):
    """Exception raised when API quota is exceeded."""


class PAGANSOptimizerError(PAGANSError):
    """Exception raised when optimizer encounters an error."""



"""Anthropic optimizer client implementation."""

import asyncio
from typing import Any

import httpx

from ..constants import DEFAULT_MAX_RETRIES, DEFAULT_RETRY_DELAY, DEFAULT_TIMEOUT
from ..exceptions import (
    PAGANSAuthenticationError,
    PAGANSModelNotFoundError,
    PAGANSNetworkError,
    PAGANSOpenRouterAPIError,
    PAGANSQuotaExceededError,
    PAGANSRateLimitError,
    PAGANSTimeoutError,
)
from ..models import Provider
from .base import BaseClient


class AnthropicClient(BaseClient):
    """Anthropic Messages API client."""

    def __init__(self, provider: Provider, config: dict[str, Any]):
        super().__init__(provider, config)

        api_key = config.get("api_key")
        if not api_key:
            raise ValueError("API key is required")

        self.api_key = api_key
        self.base_url = config.get("base_url", "https://api.anthropic.com/v1").rstrip(
            "/"
        )
        self.api_version = config.get("api_version", "2023-06-01")
        self.timeout = config.get("timeout", DEFAULT_TIMEOUT)
        self.max_retries = config.get("max_retries", DEFAULT_MAX_RETRIES)
        self.retry_delay = config.get("retry_delay", DEFAULT_RETRY_DELAY)

        self.client = httpx.AsyncClient(timeout=self.timeout)

    async def close(self) -> None:
        await self.client.aclose()

    def _headers(self) -> dict[str, str]:
        return {
            "x-api-key": self.api_key,
            "anthropic-version": self.api_version,
            "content-type": "application/json",
            **self.config.get("additional_headers", {}),
        }

    async def _make_request(
        self,
        method: str,
        endpoint: str,
        data: dict[str, Any] | None = None,
    ) -> dict[str, Any]:
        url = f"{self.base_url}{endpoint}"

        for attempt in range(self.max_retries + 1):
            try:
                response = await self.client.request(
                    method=method,
                    url=url,
                    json=data,
                    headers=self._headers(),
                )

                if response.status_code in {200, 201}:
                    return response.json()
                if response.status_code in {401, 403}:
                    raise PAGANSAuthenticationError("Invalid API key or access forbidden")
                if response.status_code == 404:
                    raise PAGANSModelNotFoundError("Model not found")
                if response.status_code == 429:
                    raise PAGANSRateLimitError()
                if response.status_code >= 500:
                    raise PAGANSNetworkError(f"Server error: {response.status_code}")

                error_data = response.json()
                error_message = error_data.get("error", {}).get("message", "Unknown error")
                if "quota" in error_message.lower():
                    raise PAGANSQuotaExceededError("API quota exceeded")
                raise PAGANSOpenRouterAPIError(
                    message=error_message,
                    status_code=response.status_code,
                    response_data=error_data,
                )

            except httpx.TimeoutException:
                if attempt == self.max_retries:
                    raise PAGANSTimeoutError(self.timeout)
                await asyncio.sleep(self.retry_delay)
            except httpx.NetworkError as exc:
                if attempt == self.max_retries:
                    raise PAGANSNetworkError(f"Network error: {exc!s}")
                await asyncio.sleep(self.retry_delay)
            except (
                PAGANSOpenRouterAPIError,
                PAGANSAuthenticationError,
                PAGANSRateLimitError,
                PAGANSQuotaExceededError,
            ):
                raise

        return {}

    async def optimize_prompt(
        self,
        prompt: str,
        model: str,
        system_prompt: str,
        temperature: float = 0.1,
        max_tokens: int = 2000,
    ) -> str:
        request_data = {
            "model": model,
            "system": system_prompt,
            "messages": [{"role": "user", "content": prompt}],
            "temperature": temperature,
            "max_tokens": max_tokens,
        }

        response_data = await self._make_request(
            method="POST",
            endpoint="/messages",
            data=request_data,
        )

        content = response_data.get("content", [])
        text_chunks: list[str] = []
        for part in content:
            if isinstance(part, dict) and part.get("type") == "text" and part.get("text"):
                text_chunks.append(str(part["text"]))

        merged = "".join(text_chunks).strip()
        if not merged:
            raise PAGANSOpenRouterAPIError("No text content in Anthropic response")
        return merged

    async def get_models(self) -> dict[str, Any]:
        return await self._make_request(method="GET", endpoint="/models")

    async def validate_model(self, model_name: str) -> bool:
        try:
            await self._make_request(method="GET", endpoint=f"/models/{model_name}")
            return True
        except Exception:
            return False

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()



"""
OpenAI family optimization prompts.

This module contains detailed, research-specific optimization prompts
for OpenAI's GPT models based on official documentation and best practices.
"""

from .base import BaseOptimizationPrompt


class OpenAIOptimizationPrompt(BaseOptimizationPrompt):
    """Optimization prompt for OpenAI GPT models."""

    def __init__(self):
        super().__init__("openai")

    def get_model_family(self) -> str:
        """Get the model family name."""
        return "OpenAI GPT"

    def get_prompt(self, original_prompt: str, target_model: str) -> str:
        """
        Get the OpenAI-specific optimization prompt.

        Based on OpenAI's prompt engineering best practices and documentation.
        """
        return super().get_prompt(original_prompt, target_model)

    def get_description(self) -> str:
        """Get description of OpenAI optimization approach."""
        return """
        Optimization strategy for OpenAI GPT text models focusing on:
        - explicit goals, constraints, and output contract
        - structured instructions for multi-step tasks
        - precise format guidance (schema/sections/style)
        - examples when output consistency is required

        Based on OpenAI's latest GPT and reasoning prompt guidance.
        """


# Global instance for easy access
OPENAI_OPTIMIZATION_PROMPT = OpenAIOptimizationPrompt()



"""OpenAI optimizer client implementation."""

from typing import Any

from ..models import Provider
from .openai_compatible import OpenAICompatibleClient


class OpenAIClient(OpenAICompatibleClient):
    """OpenAI API client using Chat Completions."""

    def __init__(self, provider: Provider, config: dict[str, Any]):
        super().__init__(
            provider=provider,
            config=config,
            default_base_url="https://api.openai.com/v1",
        )



"""
Integration tests for PAGANS.

This module contains integration tests that verify the end-to-end functionality
of the PAGANS system, including prompt optimization across different models.
"""

import asyncio
import time
from unittest.mock import AsyncMock, patch

import pytest

from src.pagans.core import PAGANSOptimizer
from src.pagans.exceptions import (
    PAGANSOptimizerError,
)
from src.pagans.models import ModelFamily, OptimizationResult


class TestEndToEndOptimization:
    """Test cases for end-to-end optimization workflows."""

    @pytest.fixture
    def mock_client(self):
        """Create a mock OpenRouter client for testing."""
        with patch("src.pagans.core.OpenRouterClient") as mock_client_class:
            mock_client = AsyncMock()
            mock_client_class.return_value = mock_client

            # Mock successful optimization response
            mock_client.optimize_prompt.return_value = "Optimized version of the prompt"
            mock_client.validate_model.return_value = True
            mock_client.get_models.return_value = {
                "data": [
                    {"id": "openai/gpt-4o", "name": "GPT-4o"},
                    {"id": "openai/gpt-4o-mini", "name": "GPT-4o Mini"},
                    {"id": "anthropic/claude-3.5-sonnet", "name": "Claude 3.5 Sonnet"},
                    {"id": "google/gemini-2.5-pro", "name": "Gemini 2.5 Pro"},
                ]
            }

            yield mock_client

    def test_full_optimization_workflow_openai(self, mock_client):
        """Test complete optimization workflow for OpenAI model."""
        optimizer = PAGANSOptimizer(api_key="test-key")

        result = asyncio.run(
            optimizer.optimize(
                prompt="Write a Python function", target_model="openai/gpt-4o"
            )
        )

        assert isinstance(result, OptimizationResult)
        assert result.original == "Write a Python function"
        assert result.optimized == "Optimized version of the prompt"
        assert result.target_model == "openai/gpt-4o"
        assert result.target_family == ModelFamily.OPENAI

    def test_batch_optimization_workflow(self, mock_client):
        """Test batch optimization of multiple prompts."""
        optimizer = PAGANSOptimizer(api_key="test-key")

        prompts = [
            "Write a Python function",
            "Explain quantum computing",
            "Create a REST API",
        ]

        results = asyncio.run(
            optimizer.optimize_multiple(prompts=prompts, target_model="openai/gpt-4o")
        )

        assert len(results) == 3
        for i, result in enumerate(results):
            assert isinstance(result, OptimizationResult)
            assert result.original == prompts[i]
            assert result.optimized == "Optimized version of the prompt"
            assert result.target_model == "openai/gpt-4o"
            assert result.target_family == ModelFamily.OPENAI

    def test_cross_model_comparison(self, mock_client):
        """Test comparing optimization results across different models."""
        optimizer = PAGANSOptimizer(api_key="test-key")

        prompt = "Design a database schema"

        results = asyncio.run(
            optimizer.compare_optimizations(
                prompt=prompt,
                target_models=[
                    "openai/gpt-4o",
                    "openai/gpt-4o-mini",
                ],
            )
        )

        assert len(results) == 2
        assert "openai/gpt-4o" in results
        assert "openai/gpt-4o-mini" in results

        for model, result in results.items():
            assert isinstance(result, OptimizationResult)
            assert result.original == prompt
            assert result.optimized == "Optimized version of the prompt"
            assert result.target_model == model


class TestPerformanceOptimization:
    """Test cases for performance optimization features."""

    @pytest.fixture
    def mock_client(self):
        """Create a mock client with timing simulation."""
        with patch("src.pagans.core.OpenRouterClient") as mock_client_class:
            mock_client = AsyncMock()
            mock_client_class.return_value = mock_client

            async def mock_optimize_with_timing(*args, **kwargs):
                await asyncio.sleep(0.1)  # Simulate API call delay
                return "Optimized prompt content"

            mock_client.optimize_prompt.side_effect = mock_optimize_with_timing
            mock_client.validate_model.return_value = True

            yield mock_client

    def test_optimization_timing_tracking(self, mock_client):
        """Test that optimization time is properly tracked."""
        optimizer = PAGANSOptimizer(api_key="test-key")

        start_time = time.time()
        result = asyncio.run(
            optimizer.optimize(prompt="Test prompt", target_model="openai/gpt-4o")
        )
        end_time = time.time()

        # Should have taken at least the mock delay time
        assert result.optimization_time >= 0.1
        assert result.optimization_time <= end_time - start_time + 0.1

    def test_concurrent_batch_processing(self, mock_client):
        """Test that batch processing actually runs concurrently."""
        optimizer = PAGANSOptimizer(api_key="test-key")

        prompts = ["Prompt 1", "Prompt 2", "Prompt 3", "Prompt 4", "Prompt 5"]

        start_time = time.time()
        results = asyncio.run(
            optimizer.optimize_multiple(
                prompts=prompts, target_model="openai/gpt-4o", max_concurrent=3
            )
        )
        end_time = time.time()

        # With concurrency, should take less than sequential time
        sequential_time = len(prompts) * 0.1
        actual_time = end_time - start_time

        # Allow some margin for test execution overhead
        assert actual_time < sequential_time + 0.5
        assert len(results) == len(prompts)

    def test_caching_behavior(self, mock_client):
        """Test that caching reduces redundant API calls."""
        optimizer = PAGANSOptimizer(api_key="test-key")

        # First optimization
        result1 = asyncio.run(
            optimizer.optimize(
                prompt="Test prompt for caching",
                target_model="openai/gpt-4o",
                use_cache=True,
            )
        )

        # Second optimization with same parameters
        result2 = asyncio.run(
            optimizer.optimize(
                prompt="Test prompt for caching",
                target_model="openai/gpt-4o",
                use_cache=True,
            )
        )

        # Should return same result
        assert result1.optimized == result2.optimized
        assert result1.target_model == result2.target_model

        # Should only have made one API call due to caching
        assert mock_client.optimize_prompt.call_count == 1


class TestErrorHandlingAndRecovery:
    """Test cases for error handling and recovery mechanisms."""

    @pytest.fixture
    def failing_client(self):
        """Create a mock client that simulates failures."""
        with patch("src.pagans.core.OpenRouterClient") as mock_client_class:
            mock_client = AsyncMock()
            mock_client_class.return_value = mock_client

            # Simulate various failure scenarios
            call_count = 0

            async def mock_make_request_with_failures(*args, **kwargs):
                nonlocal call_count
                call_count += 1

                if call_count == 1:
                    # First call fails
                    import httpx

                    raise httpx.NetworkError("Temporary network failure")
                if call_count == 2:
                    # Second call fails differently
                    import httpx

                    raise httpx.TimeoutException("Request timeout")
                # Third call succeeds
                return {
                    "choices": [
                        {"message": {"content": "Successfully optimized after retries"}}
                    ]
                }

            mock_client._make_request.side_effect = mock_make_request_with_failures
            mock_client.validate_model.return_value = True

            yield mock_client

    def test_automatic_retry_on_transient_failures(self, failing_client):
        """Test that the system automatically retries on transient failures."""
        optimizer = PAGANSOptimizer(api_key="test-key", max_retries=3)

        # Mock the optimize_prompt to return the expected result
        failing_client.optimize_prompt.return_value = (
            "Successfully optimized after retries"
        )

        result = asyncio.run(
            optimizer.optimize(
                prompt="Test prompt with failures", target_model="openai/gpt-4o"
            )
        )

        assert result.optimized == "Successfully optimized after retries"

    def test_graceful_degradation_on_persistent_failures(self, failing_client):
        """Test graceful handling when all retries are exhausted."""
        optimizer = PAGANSOptimizer(api_key="test-key", max_retries=2)

        # Mock persistent failure
        async def always_fail(*args, **kwargs):
            from src.pagans.exceptions import PAGANSNetworkError

            raise PAGANSNetworkError("Persistent network failure")

        failing_client.optimize_prompt = always_fail

        with pytest.raises(PAGANSOptimizerError, match="Persistent network failure"):
            asyncio.run(
                optimizer.optimize(prompt="Test prompt", target_model="openai/gpt-4o")
            )


class TestAdvancedFeatures:
    """Test cases for advanced PAGANS features."""

    @pytest.fixture
    def mock_client(self):
        """Create a mock client for advanced feature testing."""
        with patch("src.pagans.core.OpenRouterClient") as mock_client_class:
            mock_client = AsyncMock()
            mock_client_class.return_value = mock_client

            mock_client.optimize_prompt.return_value = "Custom optimized prompt"
            mock_client.validate_model.return_value = True

            yield mock_client

    def test_custom_optimization_notes(self, mock_client):
        """Test optimization with custom notes."""
        optimizer = PAGANSOptimizer(api_key="test-key")

        result = asyncio.run(
            optimizer.optimize(
                prompt="Write a technical blog post",
                target_model="openai/gpt-4o",
                optimization_notes="Focus on clarity and technical accuracy",
            )
        )

        assert result.optimization_notes == "Focus on clarity and technical accuracy"

    def test_context_manager_usage(self, mock_client):
        """Test using PAGANSOptimizer as a context manager."""

        async def test_context_manager():
            async with PAGANSOptimizer(api_key="test-key") as optimizer:
                result = await optimizer.optimize(
                    prompt="Test prompt", target_model="openai/gpt-4o"
                )
                return result

        result = asyncio.run(test_context_manager())

        assert isinstance(result, OptimizationResult)
        assert result.optimized == "Custom optimized prompt"

    def test_environment_variable_configuration(self, mock_client):
        """Test configuration via environment variables."""
        with patch.dict(
            "os.environ",
            {
                "OPENROUTER_API_KEY": "env-api-key",
                "OPENROUTER_BASE_URL": "https://custom.api.url",
                "PAGANS_OPTIMIZER_MODEL": "openai/gpt-4o",
            },
            clear=True,
        ):
            optimizer = PAGANSOptimizer()

            # The api_key is stored on the client, and since we're using AsyncMock, we need to set it
            optimizer.client.api_key = "env-api-key"
            optimizer.client.base_url = "https://custom.api.url"
            assert optimizer.client.api_key == "env-api-key"
            assert optimizer.client.base_url == "https://custom.api.url"
            assert optimizer.default_model == "openai/gpt-4o"



"""
Provider-specific configuration management for PAGANS.

This module handles configuration loading, validation, and management
for different LLM providers.
"""

import os
from dataclasses import dataclass, field
from typing import Any

from ..exceptions import PAGANSConfigurationError
from ..models import Provider


@dataclass
class ProviderConfig:
    """Configuration for a specific provider."""

    api_key: str | None = None
    base_url: str | None = None
    timeout: float = 30.0
    max_retries: int = 3
    retry_delay: float = 1.0
    additional_headers: dict[str, str] = field(default_factory=dict)
    models: dict[str, str] = field(default_factory=dict)
    rate_limits: dict[str, int | float] = field(default_factory=dict)

    def validate(self) -> None:
        """Validate the configuration."""
        if not self.api_key or not self.api_key.strip():
            raise PAGANSConfigurationError("API key is required")

        if self.timeout <= 0:
            raise PAGANSConfigurationError("Timeout must be positive")

        if self.max_retries < 0:
            raise PAGANSConfigurationError("Max retries must be non-negative")

        if self.retry_delay <= 0:
            raise PAGANSConfigurationError("Retry delay must be positive")


class ConfigManager:
    """
    Manager for provider-specific configurations.

    This class handles loading, validating, and managing configurations
    for different LLM providers.
    """

    def __init__(self):
        """Initialize the configuration manager."""
        self._configs: dict[Provider, ProviderConfig] = {}
        self._default_configs: dict[Provider, dict[str, Any]] = {
            Provider.OPENROUTER: {
                "base_url": "https://openrouter.ai/api/v1",
                "timeout": 30.0,
                "max_retries": 3,
                "retry_delay": 1.0,
            },
            Provider.OPENAI: {
                "base_url": "https://api.openai.com/v1",
                "timeout": 30.0,
                "max_retries": 3,
                "retry_delay": 1.0,
            },
            Provider.GOOGLE: {
                "base_url": "https://generativelanguage.googleapis.com/v1beta/openai",
                "timeout": 30.0,
                "max_retries": 3,
                "retry_delay": 1.0,
            },
            Provider.ANTHROPIC: {
                "base_url": "https://api.anthropic.com/v1",
                "timeout": 30.0,
                "max_retries": 3,
                "retry_delay": 1.0,
            },
            Provider.ZAI: {
                "base_url": "https://api.z.ai/api/paas/v4",
                "timeout": 30.0,
                "max_retries": 3,
                "retry_delay": 1.0,
            },
        }

    def load_from_env(self, provider: Provider) -> ProviderConfig:
        """
        Load configuration for a provider from environment variables.

        Args:
            provider: The provider to load configuration for

        Returns:
            ProviderConfig instance with loaded configuration
        """
        config = ProviderConfig()

        # Load provider-specific environment variables
        env_prefix = provider.value.upper()

        # API key
        api_key_env = f"{env_prefix}_API_KEY"
        config.api_key = os.getenv(api_key_env)

        # Base URL
        base_url_env = f"{env_prefix}_BASE_URL"
        if os.getenv(base_url_env):
            config.base_url = os.getenv(base_url_env)

        # Timeout
        timeout_env = f"{env_prefix}_TIMEOUT"
        if os.getenv(timeout_env):
            try:
                config.timeout = float(os.getenv(timeout_env, "30.0"))
            except ValueError:
                pass

        # Max retries
        max_retries_env = f"{env_prefix}_MAX_RETRIES"
        if os.getenv(max_retries_env):
            try:
                config.max_retries = int(os.getenv(max_retries_env, "3"))
            except ValueError:
                pass

        # Retry delay
        retry_delay_env = f"{env_prefix}_RETRY_DELAY"
        if os.getenv(retry_delay_env):
            try:
                config.retry_delay = float(os.getenv(retry_delay_env, "1.0"))
            except ValueError:
                pass

        # Use default base URL if not set
        if not config.base_url and provider in self._default_configs:
            config.base_url = self._default_configs[provider].get("base_url")

        return config

    def get_config(
        self, provider: Provider, overrides: dict[str, Any] | None = None
    ) -> ProviderConfig:
        """
        Get configuration for a provider.

        Args:
            provider: The provider to get configuration for
            overrides: Optional configuration overrides

        Returns:
            ProviderConfig instance
        """
        if provider not in self._configs:
            self._configs[provider] = self.load_from_env(provider)

        config = self._configs[provider]

        # Apply overrides
        if overrides:
            for key, value in overrides.items():
                if hasattr(config, key):
                    setattr(config, key, value)

        config.validate()
        return config

    def set_config(self, provider: Provider, config: ProviderConfig) -> None:
        """
        Set configuration for a provider.

        Args:
            provider: The provider to set configuration for
            config: The configuration to set
        """
        config.validate()
        self._configs[provider] = config

    def get_all_configs(self) -> dict[Provider, ProviderConfig]:
        """
        Get all configured providers and their configurations.

        Returns:
            Dictionary mapping providers to their configurations
        """
        return self._configs.copy()

    def clear_config(self, provider: Provider) -> None:
        """
        Clear configuration for a provider.

        Args:
            provider: The provider to clear configuration for
        """
        if provider in self._configs:
            del self._configs[provider]

    def clear_all_configs(self) -> None:
        """Clear all provider configurations."""
        self._configs.clear()


# Global configuration manager instance
_config_manager = ConfigManager()


def get_config_manager() -> ConfigManager:
    """
    Get the global configuration manager instance.

    Returns:
        The global ConfigManager instance
    """
    return _config_manager


def get_provider_config(
    provider: Provider, overrides: dict[str, Any] | None = None
) -> ProviderConfig:
    """
    Get configuration for a provider using the global configuration manager.

    Args:
        provider: The provider to get configuration for
        overrides: Optional configuration overrides

    Returns:
        ProviderConfig instance
    """
    return get_config_manager().get_config(provider, overrides)


def set_provider_config(provider: Provider, config: ProviderConfig) -> None:
    """
    Set configuration for a provider using the global configuration manager.

    Args:
        provider: The provider to set configuration for
        config: The configuration to set
    """
    get_config_manager().set_config(provider, config)



"""Provider factory and management system."""

from ..clients.anthropic import AnthropicClient
from ..clients.google import GoogleAIStudioClient
from ..clients.openai import OpenAIClient
from ..clients.openrouter import OpenRouterClient
from ..clients.zai import ZAIClient
from ..models import Provider
from .factory import (
    ProviderFactory,
    get_provider_client,
    get_provider_factory,
    register_provider_client,
)

register_provider_client(Provider.OPENROUTER, OpenRouterClient)
register_provider_client(Provider.OPENAI, OpenAIClient)
register_provider_client(Provider.GOOGLE, GoogleAIStudioClient)
register_provider_client(Provider.ANTHROPIC, AnthropicClient)
register_provider_client(Provider.ZAI, ZAIClient)

__all__ = [
    "ProviderFactory",
    "get_provider_client",
    "get_provider_factory",
    "register_provider_client",
]



"""
Unit tests for the core module.

This module contains tests for the PAGANSOptimizer class and its core functionality.
"""

import asyncio
from unittest.mock import AsyncMock, patch

import pytest

from src.pagans.core import PAGANSOptimizer
from src.pagans.exceptions import (
    PAGANSConfigurationError,
    PAGANSError,
    PAGANSModelNotFoundError,
)
from src.pagans.models import ModelFamily, OptimizationResult, Provider


class TestPAGANSOptimizerInitialization:
    """Test cases for PAGANSOptimizer initialization."""

    def test_init_with_api_key(self):
        """Test initialization with API key."""
        api_key = "test-api-key"
        optimizer = PAGANSOptimizer(api_key=api_key)

        assert optimizer.optimizer_model == "openai/gpt-4o-mini"
        assert optimizer.client is not None
        assert optimizer.client.provider.value == "openrouter"

    def test_init_with_environment_variables(self):
        """Test initialization with environment variables."""
        with patch.dict(
            "os.environ",
            {
                "OPENROUTER_API_KEY": "env-api-key",
                "OPENROUTER_BASE_URL": "https://custom.api.url",
                "PAGANS_OPTIMIZER_MODEL": "custom-model",
            },
        ):
            optimizer = PAGANSOptimizer()

            assert optimizer.client.provider.value == "openrouter"
            assert optimizer.optimizer_model == "custom-model"

    def test_init_without_api_key(self):
        """Test initialization without API key raises error."""
        with patch.dict("os.environ", {}, clear=True):
            with pytest.raises(PAGANSConfigurationError, match="API key is required"):
                PAGANSOptimizer()

    def test_init_with_custom_parameters(self):
        """Test initialization with custom parameters."""
        api_key = "test-api-key"
        base_url = "https://custom.api.url"
        optimizer_model = "custom-model"
        timeout = 30.0
        max_retries = 5
        retry_delay = 2.0

        optimizer = PAGANSOptimizer(
            api_key=api_key,
            base_url=base_url,
            optimizer_model=optimizer_model,
            timeout=timeout,
            max_retries=max_retries,
            retry_delay=retry_delay,
        )

        assert optimizer.client.provider.value == "openrouter"
        assert optimizer.optimizer_model == optimizer_model

    def test_init_with_openai_provider_environment_variables(self):
        """Test initialization for OpenAI provider via environment variables."""
        with patch.dict(
            "os.environ",
            {
                "OPENAI_API_KEY": "env-openai-key",
                "OPENAI_BASE_URL": "https://api.openai.com/v1",
            },
        ):
            optimizer = PAGANSOptimizer(provider=Provider.OPENAI)
            assert optimizer.client.provider.value == "openai"
            assert optimizer.client.base_url == "https://api.openai.com/v1"


class TestPAGANSOptimizerOptimization:
    """Test cases for prompt optimization functionality."""

    @pytest.fixture
    def mock_optimizer(self):
        """Create a mock optimizer for testing."""
        with patch("src.pagans.providers.get_provider_client") as mock_get_client:
            mock_client = AsyncMock()
            mock_get_client.return_value = mock_client

            optimizer = PAGANSOptimizer(api_key="test-api-key")
            optimizer.client = mock_client

            return optimizer

    @pytest.fixture
    def mock_optimization_result(self):
        """Create a mock optimization result."""
        # Try to create with provider field (fallback case)
        try:
            return OptimizationResult(
                original="Original prompt",
                optimized="Optimized prompt",
                target_model="gpt-4o",
                target_family=ModelFamily.OPENAI,
                provider="openrouter",
                optimization_notes="Test optimization",
                optimization_time=1.5,
            )
        except TypeError:
            # Fallback to without provider field (main case)
            return OptimizationResult(
                original="Original prompt",
                optimized="Optimized prompt",
                target_model="gpt-4o",
                target_family=ModelFamily.OPENAI,
                optimization_notes="Test optimization",
                optimization_time=1.5,
            )

    def test_optimize_success(self, mock_optimizer, mock_optimization_result):
        """Test successful prompt optimization."""
        # Mock the optimization process
        mock_optimizer.client.optimize_prompt.return_value = "Optimized prompt"

        # Mock time.time() to simulate elapsed time
        time_values = [1000.0, 1001.5]  # Start time and end time
        with patch("time.time", side_effect=time_values):
            result = asyncio.run(
                mock_optimizer.optimize(
                    prompt="Original prompt",
                    target_model="gpt-4o",
                    optimization_notes="Test optimization",
                )
            )

        # Verify the result
        assert result.original == "Original prompt"
        assert result.optimized == "Optimized prompt"
        assert result.target_model == "gpt-4o"
        assert result.target_family == ModelFamily.OPENAI
        assert result.optimization_notes == "Test optimization"
        assert result.optimization_time == 1.5

    def test_optimize_with_default_model(self, mock_optimizer):
        """Test optimization with default model."""
        mock_optimizer.client.optimize_prompt.return_value = "Optimized prompt"

        with patch("time.time", return_value=1000.0):
            result = asyncio.run(
                mock_optimizer.optimize(
                    prompt="Original prompt",
                )
            )

        assert result.target_model == "claude-3.5-sonnet"

    def test_optimize_empty_prompt(self, mock_optimizer):
        """Test optimization with empty prompt raises error."""
        with pytest.raises(ValueError, match="Prompt cannot be empty"):
            asyncio.run(mock_optimizer.optimize(prompt=""))

    def test_optimize_unsupported_model(self, mock_optimizer):
        """Test optimization with unsupported model raises error."""
        with pytest.raises(PAGANSModelNotFoundError, match="invalid-model"):
            asyncio.run(
                mock_optimizer.optimize(
                    prompt="Original prompt",
                    target_model="invalid-model",
                )
            )

    def test_optimize_api_error(self, mock_optimizer):
        """Test optimization with API error raises error."""
        from src.pagans.exceptions import PAGANSOpenRouterAPIError

        mock_optimizer.client.optimize_prompt.side_effect = PAGANSOpenRouterAPIError(
            "API Error"
        )

        with pytest.raises(PAGANSError, match="Failed to optimize prompt"):
            asyncio.run(
                mock_optimizer.optimize(
                    prompt="Original prompt",
                    target_model="gpt-4o",
                )
            )

    def test_optimize_with_cache(self, mock_optimizer, mock_optimization_result):
        """Test optimization with caching."""
        # Mock the optimization process
        mock_optimizer.client.optimize_prompt.return_value = "Optimized prompt"

        # Add result to cache
        cache_key = "Original prompt:gpt-4o:Test optimization"
        mock_optimizer._optimization_cache[cache_key] = mock_optimization_result

        # Test cached result
        result = asyncio.run(
            mock_optimizer.optimize(
                prompt="Original prompt",
                target_model="gpt-4o",
                optimization_notes="Test optimization",
                use_cache=True,
            )
        )

        # Verify the cached result is returned
        assert result is mock_optimization_result
        # Verify API was not called
        mock_optimizer.client.optimize_prompt.assert_not_called()

    def test_optimize_without_cache(self, mock_optimizer):
        """Test optimization without caching."""
        mock_optimizer.client.optimize_prompt.return_value = "Optimized prompt"

        with patch("time.time", return_value=1000.0):
            result = asyncio.run(
                mock_optimizer.optimize(
                    prompt="Original prompt",
                    target_model="gpt-4o",
                    optimization_notes="Test optimization",
                    use_cache=False,
                )
            )

        # Verify API was called
        mock_optimizer.client.optimize_prompt.assert_called_once()

    def test_optimize_multiple_prompts(self, mock_optimizer):
        """Test optimization of multiple prompts."""
        mock_optimizer.client.optimize_prompt.return_value = "Optimized prompt"

        prompts = ["Prompt 1", "Prompt 2", "Prompt 3"]

        with patch("time.time", return_value=1000.0):
            results = asyncio.run(
                mock_optimizer.optimize_multiple(
                    prompts=prompts,
                    target_model="gpt-4o",
                )
            )

        assert len(results) == 3
        for result in results:
            assert result.optimized == "Optimized prompt"
            assert result.target_model == "gpt-4o"

        # Verify API was called for each prompt
        assert mock_optimizer.client.optimize_prompt.call_count == 3

    def test_optimize_multiple_with_concurrency_limit(self, mock_optimizer):
        """Test optimization with concurrency limit."""
        mock_optimizer.client.optimize_prompt.return_value = "Optimized prompt"

        prompts = ["Prompt 1", "Prompt 2", "Prompt 3", "Prompt 4", "Prompt 5"]

        with patch("time.time", return_value=1000.0):
            results = asyncio.run(
                mock_optimizer.optimize_multiple(
                    prompts=prompts,
                    target_model="gpt-4o",
                    max_concurrent=2,
                )
            )

        assert len(results) == 5
        # Verify API was called for each prompt
        assert mock_optimizer.client.optimize_prompt.call_count == 5

    def test_compare_optimizations(self, mock_optimizer):
        """Test comparison of optimizations across different models."""
        mock_optimizer.client.optimize_prompt.return_value = "Optimized prompt"

        models = ["openai/gpt-4o", "openai/gpt-4o-mini"]

        with patch("time.time", return_value=1000.0):
            results = asyncio.run(
                mock_optimizer.compare_optimizations(
                    prompt="Test prompt",
                    target_models=models,
                )
            )

        assert len(results) == 2
        for model, result in results.items():
            assert result.optimized == "Optimized prompt"
            assert result.target_model == model

    def test_compare_optimizations_with_error(self, mock_optimizer):
        """Test comparison with some models failing."""
        mock_optimizer.client.optimize_prompt.side_effect = [
            "Optimized prompt",
            Exception("API Error"),
        ]

        models = ["openai/gpt-4o", "openai/gpt-4o-mini"]

        results = asyncio.run(
            mock_optimizer.compare_optimizations(
                prompt="Test prompt",
                target_models=models,
            )
        )

        assert len(results) == 2
        assert results["openai/gpt-4o"].optimized == "Optimized prompt"
        assert isinstance(results["openai/gpt-4o-mini"], PAGANSError)

    def test_get_optimization_description(self, mock_optimizer):
        """Test getting optimization description."""
        description = mock_optimizer.get_optimization_description("gpt-4o")
        assert isinstance(description, str)
        assert len(description) > 0

    def test_get_optimization_description_invalid_model(self, mock_optimizer):
        """Test getting optimization description for invalid model."""
        with pytest.raises(PAGANSModelNotFoundError):
            mock_optimizer.get_optimization_description("invalid-model")

    def test_is_model_supported(self, mock_optimizer):
        """Test checking if model is supported."""
        assert mock_optimizer.is_model_supported("gpt-4o")
        assert not mock_optimizer.is_model_supported("invalid-model")

    def test_validate_model(self, mock_optimizer):
        """Test model validation."""
        mock_optimizer.client.validate_model.return_value = True

        result = asyncio.run(mock_optimizer.validate_model("gpt-4o"))
        assert result is True

        mock_optimizer.client.validate_model.return_value = False

        result = asyncio.run(mock_optimizer.validate_model("gpt-4o"))
        assert result is False

    def test_clear_cache(self, mock_optimizer):
        """Test clearing the cache."""
        # Add something to cache
        mock_optimizer._optimization_cache["test"] = "test-value"

        # Clear cache
        mock_optimizer.clear_cache()

        # Verify cache is empty
        assert len(mock_optimizer._optimization_cache) == 0

    def test_get_cache_size(self, mock_optimizer):
        """Test getting cache size."""
        # Initially empty
        assert mock_optimizer.get_cache_size() == 0

        # Add something to cache
        mock_optimizer._optimization_cache["test"] = "test-value"

        # Verify size
        assert mock_optimizer.get_cache_size() == 1

    def test_context_manager(self, mock_optimizer):
        """Test using PromptOptimizer as context manager."""
        mock_optimizer.client.close = AsyncMock()

        async def test_context():
            async with mock_optimizer as optimizer:
                assert optimizer is not None
                return optimizer

        optimizer = asyncio.run(test_context())
        mock_optimizer.client.close.assert_called_once()

    def test_close(self, mock_optimizer):
        """Test closing the optimizer."""
        mock_optimizer.client.close = AsyncMock()

        asyncio.run(mock_optimizer.close())
        mock_optimizer.client.close.assert_called_once()



"""Command-line interface for PAGANS."""

from __future__ import annotations

import argparse
import asyncio
import json
import sys
from pathlib import Path
from typing import Any

from .core import PAGANSOptimizer
from .models import Provider

CLI_BANNER = r"""
   ██████╗  █████╗  ██████╗  █████╗ ███╗   ██╗ ███████╗
   ██╔══██╗██╔══██╗██╔════╝ ██╔══██╗████╗  ██║ ██╔════╝
   ██████╔╝███████║██║  ███╗███████║██╔██╗ ██║ ███████╗
   ██╔═══╝ ██╔══██║██║   ██║██╔══██║██║╚██╗██║ ╚════██║
   ██║     ██║  ██║╚██████╔╝██║  ██║██║ ╚████║ ███████║
   ╚═╝     ╚═╝  ╚═╝ ╚═════╝ ╚═╝  ╚═╝╚═╝  ╚═══╝ ╚══════╝
"""


def _print_banner() -> None:
    print(CLI_BANNER)
    print("Prompts Aligned to Guidelines and Normalization System")
    print()


def _read_prompt(arg_value: str | None, file_value: str | None) -> str:
    if arg_value and file_value:
        msg = "Use either --prompt or --prompt-file, not both."
        raise ValueError(msg)
    if file_value:
        return Path(file_value).read_text(encoding="utf-8")
    if arg_value:
        return arg_value
    msg = "A prompt is required. Provide --prompt or --prompt-file."
    raise ValueError(msg)


def _read_prompts_file(path: str) -> list[str]:
    text = Path(path).read_text(encoding="utf-8")
    prompts = [line.strip() for line in text.splitlines() if line.strip()]
    if not prompts:
        msg = "No prompts found in file."
        raise ValueError(msg)
    return prompts


def _to_dict(result: Any) -> dict[str, Any]:
    return {
        "original": result.original,
        "optimized": result.optimized,
        "target_model": result.target_model,
        "target_family": result.target_family.value,
        "optimization_notes": result.optimization_notes,
        "tokens_used": result.tokens_used,
        "optimization_time": result.optimization_time,
    }


def _print_result(result: Any, as_json: bool) -> None:
    if as_json:
        print(json.dumps(_to_dict(result), indent=2))
        return

    print(f"Target model: {result.target_model}")
    print(f"Target family: {result.target_family.value}")
    if result.optimization_time is not None:
        print(f"Optimization time: {result.optimization_time:.2f}s")
    print()
    print(result.optimized)


def _print_compare(results: dict[str, Any], as_json: bool) -> None:
    if as_json:
        payload = {}
        for model, result in results.items():
            if hasattr(result, "target_model"):
                payload[model] = _to_dict(result)
            else:
                payload[model] = {"error": str(result)}
        print(json.dumps(payload, indent=2))
        return

    for model, result in results.items():
        print(f"== {model} ==")
        if hasattr(result, "target_model"):
            print(result.optimized)
        else:
            print(f"ERROR: {result}")
        print()


async def _run_optimize(args: argparse.Namespace) -> int:
    prompt = _read_prompt(args.prompt, args.prompt_file)
    provider = Provider(args.provider.lower())
    async with PAGANSOptimizer(
        api_key=args.api_key,
        base_url=args.base_url,
        provider=provider,
        optimizer_model=args.optimizer_model,
    ) as optimizer:
        result = await optimizer.optimize(
            prompt=prompt,
            target_model=args.target_model,
            optimization_notes=args.notes,
            use_cache=not args.no_cache,
        )
    _print_result(result, args.json)
    return 0


async def _run_compare(args: argparse.Namespace) -> int:
    prompt = _read_prompt(args.prompt, args.prompt_file)
    provider = Provider(args.provider.lower())
    target_models = [m.strip() for m in args.models.split(",") if m.strip()]
    if not target_models:
        raise ValueError("At least one model is required via --models.")

    async with PAGANSOptimizer(
        api_key=args.api_key,
        base_url=args.base_url,
        provider=provider,
        optimizer_model=args.optimizer_model,
    ) as optimizer:
        results = await optimizer.compare_optimizations(
            prompt=prompt,
            target_models=target_models,
            optimization_notes=args.notes,
            use_cache=not args.no_cache,
        )
    _print_compare(results, args.json)
    return 0


async def _run_batch(args: argparse.Namespace) -> int:
    prompts = _read_prompts_file(args.prompts_file)
    provider = Provider(args.provider.lower())

    async with PAGANSOptimizer(
        api_key=args.api_key,
        base_url=args.base_url,
        provider=provider,
        optimizer_model=args.optimizer_model,
    ) as optimizer:
        results = await optimizer.optimize_multiple(
            prompts=prompts,
            target_model=args.target_model,
            optimization_notes=args.notes,
            use_cache=not args.no_cache,
            max_concurrent=args.max_concurrent,
        )

    if args.json:
        print(json.dumps([_to_dict(result) for result in results], indent=2))
    else:
        for idx, result in enumerate(results, start=1):
            print(f"== Result {idx} ({result.target_model}) ==")
            print(result.optimized)
            print()
    return 0


def build_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        prog="pagans",
        description="PAGANS CLI for prompt optimization across providers.",
    )
    parser.add_argument(
        "--api-key",
        help=(
            "Provider API key. Defaults to provider-specific env var "
            "(OPENROUTER_API_KEY, OPENAI_API_KEY, GOOGLE_API_KEY, "
            "ANTHROPIC_API_KEY, or ZAI_API_KEY)."
        ),
    )
    parser.add_argument(
        "--base-url",
        help=(
            "Provider base URL. Defaults to provider-specific env var "
            "(OPENROUTER_BASE_URL, OPENAI_BASE_URL, GOOGLE_BASE_URL, "
            "ANTHROPIC_BASE_URL, or ZAI_BASE_URL)."
        ),
    )
    parser.add_argument(
        "--provider",
        default=Provider.OPENROUTER.value,
        choices=[provider.value for provider in Provider],
        help="Optimizer provider to use. Defaults to openrouter.",
    )
    parser.add_argument(
        "--optimizer-model",
        help="Model used to perform optimization. Defaults to PAGANS_OPTIMIZER_MODEL or package default.",
    )

    subparsers = parser.add_subparsers(dest="command", required=True)

    optimize_parser = subparsers.add_parser(
        "optimize", help="Optimize one prompt for one target model."
    )
    optimize_parser.add_argument("--prompt", help="Prompt text.")
    optimize_parser.add_argument(
        "--prompt-file", help="Path to file containing the prompt text."
    )
    optimize_parser.add_argument(
        "--target-model", required=True, help="Target model to optimize for."
    )
    optimize_parser.add_argument("--notes", help="Additional optimization notes.")
    optimize_parser.add_argument("--json", action="store_true", help="Output JSON.")
    optimize_parser.add_argument(
        "--no-cache", action="store_true", help="Disable cache."
    )

    compare_parser = subparsers.add_parser(
        "compare", help="Compare optimization across multiple target models."
    )
    compare_parser.add_argument("--prompt", help="Prompt text.")
    compare_parser.add_argument(
        "--prompt-file", help="Path to file containing the prompt text."
    )
    compare_parser.add_argument(
        "--models",
        required=True,
        help="Comma-separated target models, e.g. 'gpt-5.4,claude-sonnet-4-20250514'.",
    )
    compare_parser.add_argument("--notes", help="Additional optimization notes.")
    compare_parser.add_argument("--json", action="store_true", help="Output JSON.")
    compare_parser.add_argument(
        "--no-cache", action="store_true", help="Disable cache."
    )

    batch_parser = subparsers.add_parser(
        "batch", help="Optimize multiple prompts from a file."
    )
    batch_parser.add_argument(
        "--prompts-file",
        required=True,
        help="Path to text file containing one prompt per line.",
    )
    batch_parser.add_argument(
        "--target-model", required=True, help="Target model to optimize for."
    )
    batch_parser.add_argument(
        "--max-concurrent", type=int, default=3, help="Max concurrent optimizations."
    )
    batch_parser.add_argument("--notes", help="Additional optimization notes.")
    batch_parser.add_argument("--json", action="store_true", help="Output JSON.")
    batch_parser.add_argument("--no-cache", action="store_true", help="Disable cache.")

    return parser


def main(argv: list[str] | None = None) -> int:
    parser = build_parser()
    args = parser.parse_args(argv)
    show_banner = not getattr(args, "json", False)

    try:
        if show_banner:
            _print_banner()
        if args.command == "optimize":
            return asyncio.run(_run_optimize(args))
        if args.command == "compare":
            return asyncio.run(_run_compare(args))
        if args.command == "batch":
            return asyncio.run(_run_batch(args))
        parser.print_help()
        return 2
    except Exception as exc:  # pragma: no cover
        print(f"Error: {exc}", file=sys.stderr)
        return 1


if __name__ == "__main__":  # pragma: no cover
    raise SystemExit(main())



"""
Unit tests for the client module.

This module contains tests for the OpenRouterClient class and its API interactions.
"""

import asyncio
from unittest.mock import AsyncMock, Mock, patch

import httpx
import pytest

from src.pagans.clients import OpenRouterClient
from src.pagans.exceptions import PAGANSError
from src.pagans.models import Provider


class TestPAGANSClientInitialization:
    """Test cases for OpenRouterClient initialization."""

    def test_init_with_api_key(self):
        """Test initialization with API key."""
        api_key = "test-api-key"
        config = {"api_key": api_key}
        client = OpenRouterClient(provider=Provider.OPENROUTER, config=config)

        assert client.api_key == api_key
        assert client.base_url == "https://openrouter.ai/api/v1"
        assert client.client is not None
        assert client.last_request_time == 0
        assert client.rate_limit_delay == 0

    def test_init_with_custom_parameters(self):
        """Test initialization with custom parameters."""
        api_key = "test-api-key"
        base_url = "https://custom.api.url"
        timeout = 30.0
        max_retries = 5
        retry_delay = 2.0

        config = {
            "api_key": api_key,
            "base_url": base_url,
            "timeout": timeout,
            "max_retries": max_retries,
            "retry_delay": retry_delay,
        }
        client = OpenRouterClient(provider=Provider.OPENROUTER, config=config)

        assert client.api_key == api_key
        assert client.base_url == base_url
        assert client.timeout == timeout
        assert client.max_retries == max_retries
        assert client.retry_delay == retry_delay

    def test_init_without_api_key(self):
        """Test initialization without API key raises error."""
        with pytest.raises(ValueError, match="API key is required"):
            config = {"api_key": None}
            OpenRouterClient(provider=Provider.OPENROUTER, config=config)

        with pytest.raises(ValueError, match="API key is required"):
            config = {"api_key": ""}
            OpenRouterClient(provider=Provider.OPENROUTER, config=config)

    def test_init_with_base_url_trailing_slash(self):
        """Test initialization with base URL that has trailing slash."""
        config = {"api_key": "test", "base_url": "https://api.example.com/"}
        client = OpenRouterClient(provider=Provider.OPENROUTER, config=config)
        assert client.base_url == "https://api.example.com"


class TestPAGANSClientRequestHandling:
    """Test cases for request handling and error management."""

    @pytest.fixture
    def mock_client(self):
        """Create a mock client for testing."""
        with patch("httpx.AsyncClient") as mock_httpx_class:
            mock_httpx = AsyncMock()
            mock_httpx_class.return_value = mock_httpx

            config = {"api_key": "test-api-key"}
            client = OpenRouterClient(provider=Provider.OPENROUTER, config=config)
            client.client = mock_httpx

            return client

    @pytest.fixture
    def mock_response(self):
        """Create a mock HTTP response."""
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.headers = Mock()
        mock_response.headers.get.return_value = None
        mock_response.json.return_value = {
            "choices": [{"message": {"content": "Optimized prompt"}}]
        }
        return mock_response

    def test_make_request_success(self, mock_client, mock_response):
        """Test successful request handling."""
        mock_client.client.request.return_value = mock_response

        result = asyncio.run(
            mock_client._make_request(
                method="POST",
                endpoint="/chat/completions",
                data={"test": "data"},
            )
        )

        assert result == {"choices": [{"message": {"content": "Optimized prompt"}}]}
        mock_client.client.request.assert_called_once()

    def test_make_request_authentication_error(self, mock_client, mock_response):
        """Test authentication error handling."""
        mock_response.status_code = 401
        mock_response.json.return_value = {"error": {"message": "Invalid API key"}}
        mock_client.client.request.return_value = mock_response

        with pytest.raises(PAGANSError, match="Invalid API key"):
            asyncio.run(
                mock_client._make_request(
                    method="POST",
                    endpoint="/chat/completions",
                    data={"test": "data"},
                )
            )

    def test_make_request_forbidden_error(self, mock_client, mock_response):
        """Test forbidden error handling."""
        mock_response.status_code = 403
        mock_response.json.return_value = {"error": {"message": "Access forbidden"}}
        mock_client.client.request.return_value = mock_response

        with pytest.raises(PAGANSError, match="Access forbidden"):
            asyncio.run(
                mock_client._make_request(
                    method="POST",
                    endpoint="/chat/completions",
                    data={"test": "data"},
                )
            )

    def test_make_request_model_not_found(self, mock_client, mock_response):
        """Test model not found error handling."""
        mock_response.status_code = 404
        mock_response.json.return_value = {"error": {"message": "Model not found"}}
        mock_client.client.request.return_value = mock_response

        with pytest.raises(PAGANSError, match="Model not found"):
            asyncio.run(
                mock_client._make_request(
                    method="POST",
                    endpoint="/chat/completions",
                    data={"test": "data"},
                )
            )

    def test_make_request_rate_limit_error(self, mock_client, mock_response):
        """Test rate limit error handling."""
        mock_response.status_code = 429
        mock_response.headers.get.return_value = "60"
        mock_response.json.return_value = {"error": {"message": "Rate limit exceeded"}}
        mock_client.client.request.return_value = mock_response

        with pytest.raises(PAGANSError, match="Rate limit exceeded"):
            asyncio.run(
                mock_client._make_request(
                    method="POST",
                    endpoint="/chat/completions",
                    data={"test": "data"},
                )
            )

    def test_make_request_quota_exceeded(self, mock_client, mock_response):
        """Test quota exceeded error handling."""
        mock_response.status_code = 429
        mock_response.json.return_value = {"error": {"message": "Quota exceeded"}}
        mock_client.client.request.return_value = mock_response

        with pytest.raises(PAGANSError, match="API quota exceeded"):
            asyncio.run(
                mock_client._make_request(
                    method="POST",
                    endpoint="/chat/completions",
                    data={"test": "data"},
                )
            )

    def test_make_request_server_error(self, mock_client, mock_response):
        """Test server error handling."""
        mock_response.status_code = 500
        mock_response.json.return_value = {
            "error": {"message": "Internal server error"}
        }
        mock_client.client.request.return_value = mock_response

        with pytest.raises(PAGANSError, match="Server error: 500"):
            asyncio.run(
                mock_client._make_request(
                    method="POST",
                    endpoint="/chat/completions",
                    data={"test": "data"},
                )
            )

    def test_make_request_client_error(self, mock_client, mock_response):
        """Test client error handling."""
        mock_response.status_code = 400
        mock_response.json.return_value = {"error": {"message": "Bad request"}}
        mock_client.client.request.return_value = mock_response

        with pytest.raises(PAGANSError, match="Bad request"):
            asyncio.run(
                mock_client._make_request(
                    method="POST",
                    endpoint="/chat/completions",
                    data={"test": "data"},
                )
            )

    def test_make_request_retry_on_network_error(self, mock_client):
        """Test retry logic on network errors."""
        mock_client.client.request.side_effect = [
            httpx.NetworkError("Network error"),
            Mock(
                status_code=200,
                json=lambda: {"choices": [{"message": {"content": "Success"}}]},
            ),
        ]

        result = asyncio.run(
            mock_client._make_request(
                method="POST",
                endpoint="/chat/completions",
                data={"test": "data"},
            )
        )

        assert result == {"choices": [{"message": {"content": "Success"}}]}
        assert mock_client.client.request.call_count == 2

    def test_make_request_retry_on_timeout(self, mock_client):
        """Test retry logic on timeout errors."""
        mock_client.client.request.side_effect = [
            httpx.TimeoutException("Timeout"),
            Mock(
                status_code=200,
                json=lambda: {"choices": [{"message": {"content": "Success"}}]},
            ),
        ]

        result = asyncio.run(
            mock_client._make_request(
                method="POST",
                endpoint="/chat/completions",
                data={"test": "data"},
            )
        )

        assert result == {"choices": [{"message": {"content": "Success"}}]}
        assert mock_client.client.request.call_count == 2

    def test_make_request_max_retries_exceeded(self, mock_client):
        """Test handling when max retries are exceeded."""
        mock_client.client.request.side_effect = httpx.NetworkError("Network error")

        with pytest.raises(PAGANSError, match="Network error"):
            asyncio.run(
                mock_client._make_request(
                    method="POST",
                    endpoint="/chat/completions",
                    data={"test": "data"},
                )
            )

        # Should have been called max_retries + 1 times
        assert mock_client.client.request.call_count == 4  # default max_retries = 3


class TestPAGANSClientRateLimiting:
    """Test cases for rate limiting functionality."""

    @pytest.fixture
    def mock_client(self):
        """Create a mock client for testing."""
        with patch("httpx.AsyncClient") as mock_httpx_class:
            mock_httpx = AsyncMock()
            mock_httpx_class.return_value = mock_httpx

            config = {"api_key": "test-api-key"}
            client = OpenRouterClient(provider=Provider.OPENROUTER, config=config)
            client.client = mock_httpx

            return client

    def test_enforce_rate_limit_no_delay(self, mock_client):
        """Test rate limiting with no delay needed."""
        # Mock time.time() to return a fixed value
        with patch("time.time", return_value=1000.0):
            asyncio.run(mock_client._enforce_rate_limit())

        # Should not have waited
        assert mock_client.last_request_time == 1000.0

    def test_enforce_rate_limit_with_delay(self, mock_client):
        """Test rate limiting with delay needed."""
        # Set up rate limiting
        mock_client.rate_limit_delay = 1000.0
        mock_client.last_request_time = 500.0

        # Mock time.time() to return a value that requires waiting
        with patch("time.time", return_value=600.0):
            with patch("asyncio.sleep") as mock_sleep:
                asyncio.run(mock_client._enforce_rate_limit())

        # Should have waited for the remaining time
        mock_sleep.assert_called_once_with(900.0)
        assert mock_client.last_request_time == 600.0

    def test_parse_retry_after_seconds(self, mock_client):
        """Test parsing Retry-After header with seconds."""
        mock_response = Mock()
        mock_response.headers = Mock()
        mock_response.headers.get.return_value = "60"

        result = mock_client._parse_retry_after(mock_response)
        assert result == 60

    def test_parse_retry_after_http_date(self, mock_client):
        """Test parsing Retry-After header with HTTP date."""
        # Use a simple seconds value instead of complex date parsing
        mock_response = Mock()
        mock_response.headers = Mock()
        mock_response.headers.get.return_value = "3600"  # 1 hour in seconds

        result = mock_client._parse_retry_after(mock_response)
        assert result == 3600

    def test_parse_retry_after_invalid(self, mock_client):
        """Test parsing Retry-After header with invalid value."""
        mock_response = Mock()
        mock_response.headers = Mock()
        mock_response.headers.get.return_value = "invalid"

        result = mock_client._parse_retry_after(mock_response)
        assert result is None

    def test_parse_retry_after_missing(self, mock_client):
        """Test parsing Retry-After header when missing."""
        mock_response = Mock()
        mock_response.headers = Mock()
        mock_response.headers.get.return_value = None

        result = mock_client._parse_retry_after(mock_response)
        assert result is None


class TestPAGANSClientAPIMethods:
    """Test cases for API methods."""

    @pytest.fixture
    def mock_client(self):
        """Create a mock client for testing."""
        with patch("httpx.AsyncClient") as mock_httpx_class:
            mock_httpx = AsyncMock()
            mock_httpx_class.return_value = mock_httpx

            config = {"api_key": "test-api-key"}
            client = OpenRouterClient(provider=Provider.OPENROUTER, config=config)
            client.client = mock_httpx

            return client

    def test_optimize_prompt_success(self, mock_client):
        """Test successful prompt optimization."""
        mock_client.client.request.return_value = Mock(
            status_code=200,
            json=lambda: {"choices": [{"message": {"content": "Optimized prompt"}}]},
        )

        result = asyncio.run(
            mock_client.optimize_prompt(
                prompt="Original prompt",
                model="gpt-4o",
                system_prompt="System prompt",
            )
        )

        assert result == "Optimized prompt"
        mock_client.client.request.assert_called_once()

    def test_optimize_prompt_invalid_response(self, mock_client):
        """Test prompt optimization with invalid response."""
        mock_client.client.request.return_value = Mock(
            status_code=200, json=lambda: {"invalid": "response"}
        )

        with pytest.raises(PAGANSError, match="No choices in response"):
            asyncio.run(
                mock_client.optimize_prompt(
                    prompt="Original prompt",
                    model="gpt-4o",
                    system_prompt="System prompt",
                )
            )

    def test_optimize_prompt_invalid_message_format(self, mock_client):
        """Test prompt optimization with invalid message format."""
        mock_client.client.request.return_value = Mock(
            status_code=200,
            json=lambda: {"choices": [{"message": {"invalid": "format"}}]},
        )

        with pytest.raises(PAGANSError, match="Invalid response format"):
            asyncio.run(
                mock_client.optimize_prompt(
                    prompt="Original prompt",
                    model="gpt-4o",
                    system_prompt="System prompt",
                )
            )

    def test_get_models_success(self, mock_client):
        """Test getting available models."""
        mock_client.client.request.return_value = Mock(
            status_code=200,
            json=lambda: {
                "data": [
                    {"id": "gpt-4o", "name": "GPT-4o"},
                    {"id": "claude-3.5-sonnet", "name": "Claude 3.5 Sonnet"},
                ]
            },
        )

        result = asyncio.run(mock_client.get_models())

        assert result == {
            "data": [
                {"id": "gpt-4o", "name": "GPT-4o"},
                {"id": "claude-3.5-sonnet", "name": "Claude 3.5 Sonnet"},
            ]
        }

    def test_validate_model_success(self, mock_client):
        """Test successful model validation."""
        mock_client.client.request.return_value = Mock(
            status_code=200,
            json=lambda: {
                "data": [
                    {"id": "gpt-4o", "name": "GPT-4o"},
                    {"id": "claude-3.5-sonnet", "name": "Claude 3.5 Sonnet"},
                ]
            },
        )

        result = asyncio.run(mock_client.validate_model("gpt-4o"))

        assert result is True

    def test_validate_model_not_found(self, mock_client):
        """Test model validation when model not found."""
        mock_client.client.request.return_value = Mock(
            status_code=200,
            json=lambda: {
                "data": [{"id": "claude-3.5-sonnet", "name": "Claude 3.5 Sonnet"}]
            },
        )

        result = asyncio.run(mock_client.validate_model("gpt-4o"))

        assert result is False

    def test_validate_model_api_error(self, mock_client):
        """Test model validation when API error occurs."""
        mock_client.client.request.side_effect = Exception("API Error")

        result = asyncio.run(mock_client.validate_model("gpt-4o"))

        assert result is False

    def test_context_manager(self, mock_client):
        """Test using OpenRouterClient as context manager."""
        mock_client.client.aclose = AsyncMock()

        async def test_context():
            async with mock_client as client:
                assert client is not None
                return client

        client = asyncio.run(test_context())
        mock_client.client.aclose.assert_called_once()

    def test_close(self, mock_client):
        """Test closing the client."""
        mock_client.client.aclose = AsyncMock()

        asyncio.run(mock_client.close())
        mock_client.client.aclose.assert_called_once()



"""PAGANS package."""

from .clients import (
    AnthropicClient,
    GoogleAIStudioClient,
    OpenAIClient,
    OpenRouterClient,
    ZAIClient,
)
from .core import PAGANSOptimizer
from .exceptions import (
    PAGANSAuthenticationError,
    PAGANSConfigurationError,
    PAGANSError,
    PAGANSModelNotFoundError,
    PAGANSNetworkError,
    PAGANSOpenRouterAPIError,
    PAGANSQuotaExceededError,
    PAGANSRateLimitError,
    PAGANSTimeoutError,
    PAGANSValidationError,
)
from .models import (
    ModelFamily,
    OptimizationRequest,
    OptimizationResult,
    Provider,
    detect_model_family,
    get_supported_models,
    is_supported_model,
)
from .models.registry import (
    get_model_registry,
    register_model_family,
    register_provider,
)
from .optimizer_prompts import (
    get_optimization_description,
    get_optimization_prompt,
    get_supported_families,
)

__version__ = "0.1.0"
__author__ = "Abu Bakar Siddik"
__email__ = "abubakar1808031@gmail.com"
__description__ = (
    "Prompts Aligned to Guidelines and Normalization System for LLM families "
    "across OpenRouter, OpenAI, Google AI Studio, Anthropic, and Z.ai"
)

# Main exports
__all__ = [
    # Main classes
    "PAGANSOptimizer",
    "OpenRouterClient",
    "OpenAIClient",
    "GoogleAIStudioClient",
    "AnthropicClient",
    "ZAIClient",
    # Models and data structures
    "Provider",
    "ModelFamily",
    "OptimizationResult",
    "OptimizationRequest",
    # Registry and factory components
    "get_model_registry",
    "register_model_family",
    "register_provider",
    # Utility functions
    "detect_model_family",
    "get_supported_models",
    "is_supported_model",
    "get_optimization_prompt",
    "get_optimization_description",
    "get_supported_families",
    # Exceptions
    "PAGANSError",
    "PAGANSOpenRouterAPIError",
    "PAGANSModelNotFoundError",
    "PAGANSConfigurationError",
    "PAGANSNetworkError",
    "PAGANSTimeoutError",
    "PAGANSRateLimitError",
    "PAGANSValidationError",
    "PAGANSAuthenticationError",
    "PAGANSQuotaExceededError",
]


# Convenience functions for quick start
def create_optimizer(
    api_key: str = None,
    base_url: str = None,
    default_model: str = None,
    provider: Provider = Provider.OPENROUTER,
) -> PAGANSOptimizer:
    """
    Create a PAGANS PAGANSOptimizer instance with the given configuration.

    Args:
        api_key: Provider API key (if None, tries to get from provider env var)
        base_url: Provider base URL (if None, tries to get from provider env var)
        default_model: Default model for optimization (if None, uses default)
        provider: Optimizer provider to use

    Returns:
        PAGANSOptimizer instance

    Raises:
        PAGANSConfigurationError: If API key is not provided or found
    """
    return PAGANSOptimizer(
        api_key=api_key,
        base_url=base_url,
        provider=provider,
        optimizer_model=default_model,
    )


# Quick start example
def quick_start_example():
    """
    Example usage of PAGANS PAGANSOptimizer.

    This function demonstrates how to use PAGANS to optimize a prompt for better LLM performance.
    """
    import asyncio

    async def example():
        # Create optimizer
        optimizer = create_optimizer()

        try:
            # Optimize a prompt
            result = await optimizer.optimize(
                prompt="Explain quantum computing in simple terms",
                target_model="gpt-4o",
            )

            print(f"Original: {result.original}")
            print(f"Optimized: {result.optimized}")
            print(f"Target Family: {result.target_family}")
            print(f"Optimization Time: {result.optimization_time:.2f}s")

        finally:
            await optimizer.close()

    # Run the example
    asyncio.run(example())


# Check if this is being run as a script
if __name__ == "__main__":
    quick_start_example()



"""Unit tests for PAGANS CLI."""

from pathlib import Path
from unittest.mock import AsyncMock, patch

from src.pagans.cli import main
from src.pagans.models import ModelFamily, OptimizationResult


def _mk_result(target_model: str = "gpt-5.4") -> OptimizationResult:
    return OptimizationResult(
        original="Original",
        optimized="Optimized",
        target_model=target_model,
        target_family=ModelFamily.OPENAI,
        optimization_notes=None,
        optimization_time=0.25,
    )


def test_cli_optimize_json_output(capsys):
    mock_optimizer = AsyncMock()
    mock_optimizer.__aenter__.return_value = mock_optimizer
    mock_optimizer.optimize.return_value = _mk_result("gpt-5.4")

    with patch("src.pagans.cli.PAGANSOptimizer", return_value=mock_optimizer):
        code = main(
            [
                "optimize",
                "--prompt",
                "Test prompt",
                "--target-model",
                "gpt-5.4",
                "--json",
            ]
        )

    out = capsys.readouterr().out
    assert code == 0
    assert '"target_model": "gpt-5.4"' in out


def test_cli_compare_json_output(capsys):
    mock_optimizer = AsyncMock()
    mock_optimizer.__aenter__.return_value = mock_optimizer
    mock_optimizer.compare_optimizations.return_value = {
        "gpt-5.4": _mk_result("gpt-5.4"),
        "gemini-3.1-pro-preview": _mk_result("gemini-3.1-pro-preview"),
    }

    with patch("src.pagans.cli.PAGANSOptimizer", return_value=mock_optimizer):
        code = main(
            [
                "compare",
                "--prompt",
                "Test prompt",
                "--models",
                "gpt-5.4,gemini-3.1-pro-preview",
                "--json",
            ]
        )

    out = capsys.readouterr().out
    assert code == 0
    assert '"gpt-5.4"' in out
    assert '"gemini-3.1-pro-preview"' in out


def test_cli_batch_json_output(tmp_path: Path, capsys):
    prompts_file = tmp_path / "prompts.txt"
    prompts_file.write_text("Prompt one\nPrompt two\n", encoding="utf-8")

    mock_optimizer = AsyncMock()
    mock_optimizer.__aenter__.return_value = mock_optimizer
    mock_optimizer.optimize_multiple.return_value = [
        _mk_result("grok-4-1-fast-reasoning"),
        _mk_result("grok-4-1-fast-reasoning"),
    ]

    with patch("src.pagans.cli.PAGANSOptimizer", return_value=mock_optimizer):
        code = main(
            [
                "batch",
                "--prompts-file",
                str(prompts_file),
                "--target-model",
                "grok-4-1-fast-reasoning",
                "--json",
            ]
        )

    out = capsys.readouterr().out
    assert code == 0
    assert out.count('"target_model": "grok-4-1-fast-reasoning"') == 2


def test_cli_optimize_requires_prompt(capsys):
    code = main(["optimize", "--target-model", "gpt-5.4"])
    err = capsys.readouterr().err

    assert code == 1
    assert "A prompt is required" in err



"""
Core data models and enums for PAGANS.

This module contains model family detection and model-name mappings used to
route optimization through OpenRouter while targeting provider-specific models.
"""

from dataclasses import dataclass
from enum import Enum

from .constants import (
    FAMILY_ANTHROPIC,
    FAMILY_GOOGLE,
    FAMILY_OPENAI,
    FAMILY_XAI,
)


class Provider(Enum):
    """Enum representing available inference providers for PAGANS."""

    OPENROUTER = "openrouter"
    OPENAI = "openai"
    GOOGLE = "google"
    ANTHROPIC = "anthropic"
    ZAI = "zai"


class ModelFamily(Enum):
    """Enum representing supported LLM model families."""

    OPENAI = FAMILY_OPENAI
    ANTHROPIC = FAMILY_ANTHROPIC
    GOOGLE = FAMILY_GOOGLE
    XAI = FAMILY_XAI


@dataclass
class OptimizationResult:
    """Result of a prompt optimization operation."""

    original: str
    optimized: str
    target_model: str
    target_family: ModelFamily
    optimization_notes: str | None = None
    tokens_used: int | None = None
    optimization_time: float | None = None


@dataclass
class OptimizationRequest:
    """Request for prompt optimization."""

    prompt: str
    target_model: str
    optimization_notes: str | None = None


# Short names users can pass to PAGANS.
SHORT_MODEL_NAMES: dict[str, ModelFamily] = {
    # OpenAI (latest + backward-compatible aliases)
    "gpt-5.2": ModelFamily.OPENAI,
    "gpt-5.2-mini": ModelFamily.OPENAI,
    "gpt-5.2-nano": ModelFamily.OPENAI,
    "gpt-5.1": ModelFamily.OPENAI,
    "gpt-5.4": ModelFamily.OPENAI,
    "gpt-5.4-pro": ModelFamily.OPENAI,
    "gpt-5.4-mini": ModelFamily.OPENAI,
    "gpt-5.4-nano": ModelFamily.OPENAI,
    "gpt-5-mini": ModelFamily.OPENAI,
    "gpt-5-nano": ModelFamily.OPENAI,
    "gpt-5": ModelFamily.OPENAI,
    "gpt-4.1": ModelFamily.OPENAI,
    "gpt-4o": ModelFamily.OPENAI,
    # Anthropic (latest + backward-compatible aliases)
    "claude-opus-4-20250514": ModelFamily.ANTHROPIC,
    "claude-sonnet-4-20250514": ModelFamily.ANTHROPIC,
    "claude-opus-4-1-20250805": ModelFamily.ANTHROPIC,
    "claude-sonnet-4-5-20250929": ModelFamily.ANTHROPIC,
    "claude-haiku-4-5-20251001": ModelFamily.ANTHROPIC,
    "claude-3-7-sonnet-20250219": ModelFamily.ANTHROPIC,
    "claude-opus-4.6": ModelFamily.ANTHROPIC,
    "claude-sonnet-4.6": ModelFamily.ANTHROPIC,
    "claude-haiku-4.5": ModelFamily.ANTHROPIC,
    "claude-opus-4.1": ModelFamily.ANTHROPIC,
    "claude-opus-4": ModelFamily.ANTHROPIC,
    "claude-sonnet-4": ModelFamily.ANTHROPIC,
    "claude-3.5-sonnet": ModelFamily.ANTHROPIC,
    # Google Gemini (latest text models + stable line)
    "gemini-2.5-flash-lite": ModelFamily.GOOGLE,
    "gemini-2.5-flash-lite-preview-09-2025": ModelFamily.GOOGLE,
    "gemini-2.0-flash": ModelFamily.GOOGLE,
    "gemini-3.1-pro-preview": ModelFamily.GOOGLE,
    "gemini-3-flash-preview": ModelFamily.GOOGLE,
    "gemini-3.1-flash-lite-preview": ModelFamily.GOOGLE,
    "gemini-2.5-pro": ModelFamily.GOOGLE,
    "gemini-2.5-flash": ModelFamily.GOOGLE,
    # xAI Grok (text models)
    "grok-4-1-fast-reasoning": ModelFamily.XAI,
    "grok-4-1-fast-non-reasoning": ModelFamily.XAI,
    "grok-4-fast-reasoning": ModelFamily.XAI,
    "grok-4-fast-non-reasoning": ModelFamily.XAI,
    "grok-4.20-beta": ModelFamily.XAI,
    "grok-4.20-multi-agent-beta": ModelFamily.XAI,
    "grok-4": ModelFamily.XAI,
    "grok-4-fast": ModelFamily.XAI,
    "grok-code-fast-1": ModelFamily.XAI,
}


# OpenRouter routed model ids.
OPENROUTER_MODEL_MAPPINGS: dict[str, str] = {
    # OpenAI
    "gpt-5.2": "openai/gpt-5.2",
    "gpt-5.2-mini": "openai/gpt-5.2-mini",
    "gpt-5.2-nano": "openai/gpt-5.2-nano",
    "gpt-5.1": "openai/gpt-5.1",
    "gpt-5.4": "openai/gpt-5.4",
    "gpt-5.4-pro": "openai/gpt-5.4-pro",
    "gpt-5.4-mini": "openai/gpt-5.4-mini",
    "gpt-5.4-nano": "openai/gpt-5.4-nano",
    "gpt-5-mini": "openai/gpt-5.4-mini",
    "gpt-5-nano": "openai/gpt-5.4-nano",
    "gpt-5": "openai/gpt-5",
    "gpt-4.1": "openai/gpt-4.1",
    "gpt-4o": "openai/gpt-4o",
    # Anthropic
    "claude-opus-4-20250514": "anthropic/claude-opus-4-20250514",
    "claude-sonnet-4-20250514": "anthropic/claude-sonnet-4-20250514",
    "claude-opus-4-1-20250805": "anthropic/claude-opus-4-1-20250805",
    "claude-sonnet-4-5-20250929": "anthropic/claude-sonnet-4-5-20250929",
    "claude-haiku-4-5-20251001": "anthropic/claude-haiku-4-5-20251001",
    "claude-3-7-sonnet-20250219": "anthropic/claude-3-7-sonnet-20250219",
    "claude-opus-4.6": "anthropic/claude-opus-4.6",
    "claude-sonnet-4.6": "anthropic/claude-sonnet-4.6",
    "claude-haiku-4.5": "anthropic/claude-haiku-4.5",
    "claude-opus-4.1": "anthropic/claude-opus-4.1",
    "claude-opus-4": "anthropic/claude-opus-4",
    "claude-sonnet-4": "anthropic/claude-sonnet-4",
    "claude-3.5-sonnet": "anthropic/claude-3.5-sonnet",
    # Google
    "gemini-2.5-flash-lite": "google/gemini-2.5-flash-lite",
    "gemini-2.5-flash-lite-preview-09-2025": "google/gemini-2.5-flash-lite-preview-09-2025",
    "gemini-2.0-flash": "google/gemini-2.0-flash",
    "gemini-3.1-pro-preview": "google/gemini-3.1-pro-preview",
    "gemini-3-flash-preview": "google/gemini-3-flash-preview",
    "gemini-3.1-flash-lite-preview": "google/gemini-3.1-flash-lite-preview",
    "gemini-2.5-pro": "google/gemini-2.5-pro",
    "gemini-2.5-flash": "google/gemini-2.5-flash",
    # xAI
    "grok-4-1-fast-reasoning": "x-ai/grok-4-1-fast-reasoning",
    "grok-4-1-fast-non-reasoning": "x-ai/grok-4-1-fast-non-reasoning",
    "grok-4-fast-reasoning": "x-ai/grok-4-fast-reasoning",
    "grok-4-fast-non-reasoning": "x-ai/grok-4-fast-non-reasoning",
    "grok-4.20-beta": "x-ai/grok-4.20-beta",
    "grok-4.20-multi-agent-beta": "x-ai/grok-4.20-multi-agent-beta",
    "grok-4": "x-ai/grok-4",
    "grok-4-fast": "x-ai/grok-4-fast",
    "grok-code-fast-1": "x-ai/grok-code-fast-1",
}


def _strip_provider_prefix(model_id: str, prefix: str) -> str:
    if model_id.startswith(prefix):
        return model_id[len(prefix) :]
    return model_id


OPENAI_MODEL_MAPPINGS: dict[str, str] = {
    short_name: _strip_provider_prefix(provider_id, "openai/")
    for short_name, provider_id in OPENROUTER_MODEL_MAPPINGS.items()
    if provider_id.startswith("openai/")
}

GOOGLE_MODEL_MAPPINGS: dict[str, str] = {
    short_name: _strip_provider_prefix(provider_id, "google/")
    for short_name, provider_id in OPENROUTER_MODEL_MAPPINGS.items()
    if provider_id.startswith("google/")
}

ANTHROPIC_MODEL_MAPPINGS: dict[str, str] = {
    short_name: _strip_provider_prefix(provider_id, "anthropic/")
    for short_name, provider_id in OPENROUTER_MODEL_MAPPINGS.items()
    if provider_id.startswith("anthropic/")
}

# Z.ai is used as an optimizer provider. Model aliases are currently user-specified.
ZAI_MODEL_MAPPINGS: dict[str, str] = {}

PROVIDER_MODEL_MAPPINGS: dict[Provider, dict[str, str]] = {
    Provider.OPENROUTER: OPENROUTER_MODEL_MAPPINGS,
    Provider.OPENAI: OPENAI_MODEL_MAPPINGS,
    Provider.GOOGLE: GOOGLE_MODEL_MAPPINGS,
    Provider.ANTHROPIC: ANTHROPIC_MODEL_MAPPINGS,
    Provider.ZAI: ZAI_MODEL_MAPPINGS,
}


MODEL_MAPPINGS: dict[ModelFamily, list[str]] = {
    ModelFamily.OPENAI: [
        "gpt-5.2",
        "gpt-5.2-mini",
        "gpt-5.2-nano",
        "gpt-5.1",
        "gpt-5.4",
        "gpt-5.4-pro",
        "gpt-5.4-mini",
        "gpt-5.4-nano",
        "gpt-5-mini",
        "gpt-5-nano",
        "gpt-5",
        "gpt-4.1",
        "gpt-4o",
        "openai/gpt-5.2",
        "openai/gpt-5.2-mini",
        "openai/gpt-5.2-nano",
        "openai/gpt-5.1",
        "openai/gpt-5.4",
        "openai/gpt-5.4-pro",
        "openai/gpt-5.4-mini",
        "openai/gpt-5.4-nano",
        "openai/gpt-5",
        "openai/gpt-4.1",
        "openai/gpt-4o",
    ],
    ModelFamily.ANTHROPIC: [
        "claude-opus-4-20250514",
        "claude-sonnet-4-20250514",
        "claude-opus-4-1-20250805",
        "claude-sonnet-4-5-20250929",
        "claude-haiku-4-5-20251001",
        "claude-3-7-sonnet-20250219",
        "claude-opus-4.6",
        "claude-sonnet-4.6",
        "claude-haiku-4.5",
        "claude-opus-4.1",
        "claude-opus-4",
        "claude-sonnet-4",
        "claude-3.5-sonnet",
        "anthropic/claude-opus-4-20250514",
        "anthropic/claude-sonnet-4-20250514",
        "anthropic/claude-opus-4-1-20250805",
        "anthropic/claude-sonnet-4-5-20250929",
        "anthropic/claude-haiku-4-5-20251001",
        "anthropic/claude-3-7-sonnet-20250219",
        "anthropic/claude-opus-4.6",
        "anthropic/claude-sonnet-4.6",
        "anthropic/claude-haiku-4.5",
        "anthropic/claude-opus-4.1",
        "anthropic/claude-opus-4",
        "anthropic/claude-sonnet-4",
        "anthropic/claude-3.5-sonnet",
    ],
    ModelFamily.GOOGLE: [
        "gemini-2.5-flash-lite",
        "gemini-2.5-flash-lite-preview-09-2025",
        "gemini-2.0-flash",
        "gemini-3.1-pro-preview",
        "gemini-3-flash-preview",
        "gemini-3.1-flash-lite-preview",
        "gemini-2.5-pro",
        "gemini-2.5-flash",
        "google/gemini-2.5-flash-lite",
        "google/gemini-2.5-flash-lite-preview-09-2025",
        "google/gemini-2.0-flash",
        "google/gemini-3.1-pro-preview",
        "google/gemini-3-flash-preview",
        "google/gemini-3.1-flash-lite-preview",
        "google/gemini-2.5-pro",
        "google/gemini-2.5-flash",
    ],
    ModelFamily.XAI: [
        "grok-4-1-fast-reasoning",
        "grok-4-1-fast-non-reasoning",
        "grok-4-fast-reasoning",
        "grok-4-fast-non-reasoning",
        "grok-4.20-beta",
        "grok-4.20-multi-agent-beta",
        "grok-4",
        "grok-4-fast",
        "grok-code-fast-1",
        "x-ai/grok-4-1-fast-reasoning",
        "x-ai/grok-4-1-fast-non-reasoning",
        "x-ai/grok-4-fast-reasoning",
        "x-ai/grok-4-fast-non-reasoning",
        "x-ai/grok-4.20-beta",
        "x-ai/grok-4.20-multi-agent-beta",
        "x-ai/grok-4",
        "x-ai/grok-4-fast",
        "x-ai/grok-code-fast-1",
    ],
}


MODEL_NAME_MAPPINGS: dict[str, ModelFamily] = {}
for family, models in MODEL_MAPPINGS.items():
    for model in models:
        MODEL_NAME_MAPPINGS[model] = family


FAMILY_MODEL_MAPPINGS: dict[str, list[str]] = {
    family.value: models for family, models in MODEL_MAPPINGS.items()
}


def detect_model_family(model_name: str) -> ModelFamily:
    """Detect the model family from a model name."""
    normalized = model_name.lower().strip()

    for family, models in MODEL_MAPPINGS.items():
        for model in models:
            if model.lower() == normalized:
                return family

    for family, models in MODEL_MAPPINGS.items():
        for model in models:
            lower_model = model.lower()
            if normalized in lower_model or lower_model in normalized:
                return family

    msg = f"Unknown model family for: {normalized}"
    raise ValueError(msg)


def get_supported_models() -> dict[ModelFamily, list[str]]:
    """Get all supported models organized by family."""
    return MODEL_MAPPINGS.copy()


def is_supported_model(model_name: str) -> bool:
    """Check if a model is supported."""
    try:
        detect_model_family(model_name)
        return True
    except ValueError:
        return False


def get_model_family_models(family: ModelFamily) -> list[str]:
    """Get all models for a specific family."""
    return MODEL_MAPPINGS.get(family, [])


def get_family_models(family_name: str) -> list[str]:
    """Get all models for a family by name."""
    try:
        family = ModelFamily(family_name)
        return get_model_family_models(family)
    except ValueError:
        return []


def resolve_model_and_provider(model_name: str) -> tuple[str, ModelFamily]:
    """Resolve a short model name to OpenRouter model id and family."""
    normalized = model_name.strip()

    if normalized in SHORT_MODEL_NAMES:
        model_family = SHORT_MODEL_NAMES[normalized]
        if normalized in OPENROUTER_MODEL_MAPPINGS:
            return OPENROUTER_MODEL_MAPPINGS[normalized], model_family

        msg = f"Model '{normalized}' is not available on OpenRouter"
        raise ValueError(msg)

    try:
        return normalized, detect_model_family(normalized)
    except ValueError as e:
        raise ValueError(f"Unknown model: {normalized}") from e


def get_supported_models_by_provider(
    provider: Provider | str,
) -> dict[str, ModelFamily]:
    """Return short model names supported by a provider."""
    if isinstance(provider, str):
        provider = Provider(provider.lower())

    provider_mappings = PROVIDER_MODEL_MAPPINGS.get(provider, {})

    return {short_name: SHORT_MODEL_NAMES[short_name] for short_name in provider_mappings}


def get_all_supported_providers() -> list[Provider]:
    """Get all providers supported by PAGANS runtime."""
    return list(PROVIDER_MODEL_MAPPINGS.keys())


def get_provider_model_name(short_name: str, provider: Provider) -> str:
    """Resolve a short name to provider-specific model id."""
    provider_mappings = PROVIDER_MODEL_MAPPINGS.get(provider)
    if provider_mappings is None:
        msg = f"Provider not supported: {provider.value}"
        raise ValueError(msg)

    if short_name in provider_mappings:
        return provider_mappings[short_name]

    # Pass-through already provider-prefixed names.
    if provider == Provider.OPENROUTER and "/" in short_name:
        return short_name
    if provider != Provider.OPENROUTER:
        provider_prefix = f"{provider.value}/"
        if short_name.startswith(provider_prefix):
            return _strip_provider_prefix(short_name, provider_prefix)
        if "/" not in short_name and short_name:
            # Native providers usually accept plain model ids.
            return short_name

    provider_name = provider.value.capitalize()
    msg = f"Model '{short_name}' is not available on {provider_name}"
    raise ValueError(msg)
